Package: alevinQC
Type: Package
Title: Generate QC Reports For Alevin Output
Version: 1.27.0
Date: 2025-07-17
Authors@R: c(person("Charlotte", "Soneson", role = c("aut", "cre"), 
                    email = "charlottesoneson@gmail.com", 
                    comment = c(ORCID = "0000-0003-3833-2169")),
    person("Avi", "Srivastava", role = "aut"),
    person("Rob", "Patro", role = "aut"),
    person("Dongze", "He", role = "aut"))
Description: Generate QC reports summarizing the output from an alevin, 
    alevin-fry, or simpleaf run. 
    Reports can be generated as html or pdf files, or as shiny applications.
Encoding: UTF-8
SystemRequirements: C++11
Depends: R (>= 4.0)
Imports: rmarkdown (>= 2.5), tools, methods, ggplot2 (>= 3.4.0),
        GGally, dplyr, rjson, shiny, shinydashboard, DT, stats, utils,
        tximport (>= 1.17.4), cowplot, rlang, Rcpp
RoxygenNote: 7.3.2
Suggests: knitr, BiocStyle, testthat (>= 3.0.0), BiocManager
VignetteBuilder: knitr
biocViews: QualityControl, SingleCell
URL: https://github.com/csoneson/alevinQC
BugReports: https://github.com/csoneson/alevinQC/issues
License: MIT + file LICENSE
Config/testthat/edition: 3
LinkingTo: Rcpp
git_url: https://git.bioconductor.org/packages/alevinQC
git_branch: devel
git_last_commit: 3024244
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
NeedsCompilation: yes
Packaged: 2025-10-30 20:01:09 UTC; biocbuild
Author: Charlotte Soneson [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3833-2169>),
  Avi Srivastava [aut],
  Rob Patro [aut],
  Dongze He [aut]
Maintainer: Charlotte Soneson <charlottesoneson@gmail.com>

# Generated by roxygen2: do not edit by hand

export(alevinFryQCReport)
export(alevinFryQCShiny)
export(alevinQCReport)
export(alevinQCShiny)
export(checkAlevinFryInputFiles)
export(checkAlevinInputFiles)
export(plotAlevinBarcodeCollapse)
export(plotAlevinHistogram)
export(plotAlevinKneeNbrGenes)
export(plotAlevinKneeRaw)
export(plotAlevinQuant)
export(plotAlevinQuantPairs)
export(readAlevinFryQC)
export(readAlevinQC)
export(simpleafQCReport)
export(simpleafQCShiny)
import(dplyr)
import(rlang)
importFrom(DT,dataTableOutput)
importFrom(DT,datatable)
importFrom(DT,renderDataTable)
importFrom(GGally,ggally_cor)
importFrom(GGally,ggally_densityDiag)
importFrom(GGally,ggally_points)
importFrom(GGally,ggpairs)
importFrom(Rcpp,sourceCpp)
importFrom(cowplot,get_legend)
importFrom(cowplot,plot_grid)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_label)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_rug)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_log10)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(methods,is)
importFrom(rjson,fromJSON)
importFrom(rlang,.data)
importFrom(rmarkdown,pandoc_available)
importFrom(rmarkdown,pandoc_exec)
importFrom(rmarkdown,render)
importFrom(shiny,actionButton)
importFrom(shiny,fluidRow)
importFrom(shiny,icon)
importFrom(shiny,observeEvent)
importFrom(shiny,plotOutput)
importFrom(shiny,renderPlot)
importFrom(shiny,shinyApp)
importFrom(shiny,stopApp)
importFrom(shinydashboard,box)
importFrom(shinydashboard,dashboardHeader)
importFrom(shinydashboard,dashboardPage)
importFrom(shinydashboard,dashboardSidebar)
importFrom(shinydashboard,dropdownMenu)
importFrom(shinydashboard,dropdownMenuOutput)
importFrom(shinydashboard,notificationItem)
importFrom(shinydashboard,renderMenu)
importFrom(tools,file_ext)
importFrom(tools,file_path_sans_ext)
importFrom(tximport,tximport)
importFrom(utils,packageVersion)
importFrom(utils,read.delim)
useDynLib(alevinQC, .registration = TRUE)

## alevinQC

[![Codecov.io coverage status](https://codecov.io/github/csoneson/alevinQC/coverage.svg?branch=master)](https://codecov.io/github/csoneson/alevinQC)
[![R build status](https://github.com/csoneson/alevinQC/workflows/R-CMD-check/badge.svg)](https://github.com/csoneson/alevinQC/actions)

The `alevinQC` R package provides functionality for generating QC reports
summarizing the output of [alevin](https://salmon.readthedocs.io/en/latest/alevin.html)
([Srivastava et al., Genome Biology 20:65 (2019)](https://genomebiology.biomedcentral.com/articles/10.1186/s13059-019-1670-y)). The reports can
be generated in html or pdf format, or as R/Shiny applications.

### Installation:

`alevinQC` is available from
[Bioconductor](https://bioconductor.org/packages/alevinQC/), and can be
installed using the `BiocManager` CRAN package:

```
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("alevinQC")
```

Note that `alevinQC` v 1.1 or newer is required to process output from Salmon version 0.14.0 or newer.

### Example usage:

```
alevinQCReport(baseDir = system.file("extdata/alevin_example_v0.14", 
                                     package = "alevinQC"),
               sampleId = "testSample", 
               outputFile = "alevinReport.html", 
               outputFormat = "html_document",
               outputDir = tempdir(), forceOverwrite = TRUE)
```

For more information, we refer to the package vignette.

![](https://raw.githubusercontent.com/csoneson/alevinQC/master/inst/extdata/alevinQC_screenshot1.png)
![](https://raw.githubusercontent.com/csoneson/alevinQC/master/inst/extdata/alevinQC_screenshot2.png)

#' Pairs plot with quantification summary statistics
#'
#' @author Charlotte Soneson
#'
#' @param cbTable \code{data.frame} (such as the \code{cbTable} returned by
#'     \code{readAlevinQC} or \code{readAlevinFryQC}) with collapsed barcode
#'     frequencies, the total UMI count and the number of detected genes
#'     for each cell.
#' @param colName Character scalar giving the name of a logical column of
#'     \code{cbTable} to use for coloring the points.
#' @param firstSelColName Character scalar indicating the name of the logical
#'     column in \code{cbTable} that corresponds to the original selection of
#'     barcodes for quantification.
#'
#' @export
#'
#' @importFrom GGally ggpairs ggally_cor ggally_points ggally_densityDiag
#' @importFrom ggplot2 aes scale_color_manual scale_fill_manual theme_bw
#' @import dplyr
#' @import rlang
#'
#' @return A ggmatrix object
#'
#' @examples
#' alevin <- readAlevinQC(system.file("extdata/alevin_example_v0.14",
#'                                    package = "alevinQC"))
#' plotAlevinQuantPairs(alevin$cbTable, colName = "inFinalWhiteList")
#'
plotAlevinQuantPairs <- function(cbTable, colName = "inFinalWhiteList",
                                 firstSelColName = "inFirstWhiteList") {
    ## Check input arguments
    .assertVector(x = cbTable, type = "data.frame")
    .assertScalar(x = colName, type = "character",
                  validValues = colnames(cbTable))
    .assertVector(x = cbTable[[colName]], type = "logical")
    .assertScalar(x = firstSelColName, type = "character",
                  validValues = colnames(cbTable))
    .assertVector(x = cbTable[[firstSelColName]], type = "logical")
    stopifnot(all(c("collapsedFreq", "totalUMICount",
                    "nbrGenesAboveZero") %in% colnames(cbTable)))

    GGally::ggpairs(
        cbTable %>% dplyr::filter(.data[[firstSelColName]]) %>%
            dplyr::rename(`Barcode frequency` = "collapsedFreq",
                          `Total UMI count` = "totalUMICount",
                          `Nbr detected genes` = "nbrGenesAboveZero"),
        mapping = ggplot2::aes(color = .data[[colName]]),
        columns = c("Barcode frequency", "Total UMI count",
                    "Nbr detected genes"),
        upper = list(continuous = function(data, mapping, ...) {
            GGally::ggally_cor(data = data, mapping = mapping) +
                ggplot2::scale_color_manual(values = c("darkgreen", "red"))}),
        lower = list(continuous = function(data, mapping, ...) {
            GGally::ggally_points(data = data, mapping = mapping, alpha = 0.5) +
                ggplot2::scale_color_manual(values = c("darkgreen", "red"))}),
        diag = list(continuous = function(data, mapping, ...) {
            GGally::ggally_densityDiag(data = data, mapping = mapping,
                                       alpha = 0.5) +
                ggplot2::scale_fill_manual(values = c("darkgreen", "red"))})
    ) +
        ggplot2::theme_bw()
}

test_that("checking for input files works", {
    ## All required files available - check should pass
    expect_equal(checkAlevinFryInputFiles(
        mapDir = system.file("extdata/alevinfry_example_v0.5.0/map",
                             package = "alevinQC"),
        permitDir = system.file("extdata/alevinfry_example_v0.5.0/permit",
                                package = "alevinQC"),
        quantDir = system.file("extdata/alevinfry_example_v0.5.0/quant",
                               package = "alevinQC")),
        "v0.5.0")

    ## Remove one file - check should fail
    tmp <- tempdir()
    file.copy(from = system.file("extdata/alevinfry_example_v0.5.0",
                                 package = "alevinQC"),
              to = tmp, overwrite = TRUE, recursive = TRUE)
    file.remove(file.path(tmp, "alevinfry_example_v0.5.0/map/cmd_info.json"))
    expect_error(checkAlevinFryInputFiles(
        mapDir = file.path(tmp, "alevinfry_example_v0.5.0", "map"),
        permitDir = file.path(tmp, "alevinfry_example_v0.5.0", "permit"),
        quantDir = file.path(tmp, "alevinfry_example_v0.5.0", "quant")))
})

## Read provided example input files for tests of file reading/plotting
alevin <- readAlevinFryQC(
    mapDir = system.file("extdata/alevinfry_example_v0.5.0/map",
                         package = "alevinQC"),
    permitDir = system.file("extdata/alevinfry_example_v0.5.0/permit",
                            package = "alevinQC"),
    quantDir = system.file("extdata/alevinfry_example_v0.5.0/quant",
                           package = "alevinQC"))

test_that("reading input files works", {
    expect_length(alevin, 4)
    expect_type(alevin, "list")
    expect_named(alevin, c("cbTable", "versionTable", "summaryTables", "type"))
    expect_equal(alevin$type, "alevin-fry")

    expect_equal(nrow(alevin$cbTable), 154362)
    expect_equal(sum(alevin$cbTable$inPermitList), 101)
    expect_equal(sum(!is.na(alevin$cbTable$mappingRate)), 101)

    expect_error(.makeSummaryTable(alevin$cbTable, colName = "collapsedFreq"),
                 "is.logical")
    expect_error(.makeSummaryTable(alevin$cbTable, colName = "missingCol"),
                 "colName")
})

test_that("reading input files gives same result as for v0.4.3", {
    cb043 <- readAlevinFryQC(
        mapDir = system.file("extdata/alevinfry_example_v0.4.3/map",
                             package = "alevinQC"),
        permitDir = system.file("extdata/alevinfry_example_v0.4.3/permit",
                                package = "alevinQC"),
        quantDir = system.file("extdata/alevinfry_example_v0.4.3/quant",
                               package = "alevinQC"))
    ## Ties can be sorted differently - can't just compare outputs directly
    expect_equal(sort(alevin$cbTable$CB), sort(cb043$cbTable$CB))
    cb043$cbTable <- cb043$cbTable[match(alevin$cbTable$CB, cb043$cbTable$CB), ]
    for (cn in setdiff(colnames(alevin$cbTable), "ranking")) {
        expect_equal(alevin$cbTable[[cn]], cb043$cbTable[[cn]])
    }
})

test_that("plots are generated", {
    expect_true(ggplot2::is_ggplot(plotAlevinKneeRaw(
        alevin$cbTable, firstSelColName = "inPermitList")))
    expect_true(ggplot2::is_ggplot(plotAlevinBarcodeCollapse(
        alevin$cbTable,
        firstSelColName = "inPermitList", countCol = "nbrMappedUMI")))
    expect_s3_class(plotAlevinQuantPairs(
        alevin$cbTable,
        firstSelColName = "inPermitList", colName = "inPermitList"), "ggmatrix")
    expect_true(ggplot2::is_ggplot(plotAlevinKneeNbrGenes(
        alevin$cbTable, firstSelColName = "inPermitList")))
    expect_true(ggplot2::is_ggplot(plotAlevinHistogram(
        alevin$cbTable, colName = "inPermitList", cbName = "permitlist",
        firstSelColName = "inPermitList")))

    expect_error(plotAlevinQuantPairs(alevin$cbTable,
                                      firstSelColName = "inPermitList",
                                      colName = "nbrGenesAboveMean"))
    expect_error(plotAlevinQuant(alevin$cbTable,
                                 firstSelColName = "inPermitList",
                                 colName = "nbrGenesAboveMean"))
    expect_error(plotAlevinHistogram(alevin$cbTable,
                                     firstSelColName = "inPermitList",
                                     colName = "missing"))
    expect_error(plotAlevinHistogram(alevin$cbTable,
                                     firstSelColName = "inPermitList",
                                     colName = "CB"))
})

tempDir <- tempdir()
if (file.exists(file.path(tempDir, "tmppfry.Rmd"))) {
    file.remove(file.path(tempDir, "tmppfry.Rmd"))
}
if (file.exists(file.path(tempDir, "tmppfry.html"))) {
    file.remove(file.path(tempDir, "tmppfry.html"))
}
if (file.exists(file.path(tempDir, "tmpp2fry.html"))) {
    file.remove(file.path(tempDir, "tmpp2fry.html"))
}
if (file.exists(file.path(tempDir, "tmppfry3.Rmd"))) {
    file.remove(file.path(tempDir, "tmppfry3.Rmd"))
}
if (file.exists(file.path(tempDir, "tmppfry3.html"))) {
    file.remove(file.path(tempDir, "tmppfry3.html"))
}
if (file.exists(file.path(tempDir, "tmppfry.pdf"))) {
    file.remove(file.path(tempDir, "tmppfry.pdf"))
}

test_that("input arguments are processed correctly", {
    ## outputFormat
    expect_error(alevinFryQCReport(
        mapDir = system.file("extdata/alevinfry_example_v0.5.0/map",
                             package = "alevinQC"),
        permitDir = system.file("extdata/alevinfry_example_v0.5.0/permit",
                                package = "alevinQC"),
        quantDir = system.file("extdata/alevinfry_example_v0.5.0/quant",
                               package = "alevinQC"),
        outputFormat = "html", outputFile = "tmppfry.html",
        outputDir = tempDir, sampleId = "test"))
    expect_error(alevinFryQCReport(
        mapDir = system.file("extdata/alevinfry_example_v0.5.0/map",
                             package = "alevinQC"),
        permitDir = system.file("extdata/alevinfry_example_v0.5.0/permit",
                                package = "alevinQC"),
        quantDir = system.file("extdata/alevinfry_example_v0.5.0/quant",
                               package = "alevinQC"),
        outputFormat = "html_document", outputFile = "tmppfry.pdf",
        outputDir = tempDir, sampleId = "test"))
    expect_error(alevinFryQCReport(
        mapDir = system.file("extdata/alevinfry_example_v0.5.0/map",
                             package = "alevinQC"),
        permitDir = system.file("extdata/alevinfry_example_v0.5.0/permit",
                                package = "alevinQC"),
        quantDir = system.file("extdata/alevinfry_example_v0.5.0/quant",
                               package = "alevinQC"),
        outputFormat = "html_document", outputFile = "tmppfry.html",
        outputDir = tempDir, sampleId = c("s1", "s2")))
})

test_that("report generation works", {
    rpt <- alevinFryQCReport(
        mapDir = system.file("extdata/alevinfry_example_v0.5.0/map",
                             package = "alevinQC"),
        permitDir = system.file("extdata/alevinfry_example_v0.5.0/permit",
                                package = "alevinQC"),
        quantDir = system.file("extdata/alevinfry_example_v0.5.0/quant",
                               package = "alevinQC"),
        sampleId = "test", outputFile = "tmppfry.html",
        outputDir = tempDir, outputFormat = NULL,
        forceOverwrite = FALSE)
    expect_equal(basename(rpt), "tmppfry.html")
    expect_error(alevinFryQCReport(
        mapDir = system.file("extdata/alevinfry_example_v0.5.0/map",
                             package = "alevinQC"),
        permitDir = system.file("extdata/alevinfry_example_v0.5.0/permit",
                                package = "alevinQC"),
        quantDir = system.file("extdata/alevinfry_example_v0.5.0/quant",
                               package = "alevinQC"),
        sampleId = "test", outputFile = "tmppfry.html",
        outputDir = tempDir, outputFormat = NULL,
        forceOverwrite = FALSE))
    file.copy(system.file("extdata/alevin_report_template.Rmd",
                          package = "alevinQC"),
              file.path(tempDir, "tmppfry.Rmd"))
    expect_warning(
        expect_error(
            alevinFryQCReport(
                mapDir = system.file("extdata/alevinfry_example_v0.5.0/map",
                                     package = "alevinQC"),
                permitDir = system.file("extdata/alevinfry_example_v0.5.0/permit",
                                        package = "alevinQC"),
                quantDir = system.file("extdata/alevinfry_example_v0.5.0/quant",
                                       package = "alevinQC"),
                sampleId = "test", outputFile = "tmppfry.html",
                outputDir = tempDir, outputFormat = NULL,
                forceOverwrite = TRUE)
        )
    )
    rpt <- alevinFryQCReport(
        mapDir = system.file("extdata/alevinfry_example_v0.5.0/map",
                             package = "alevinQC"),
        permitDir = system.file("extdata/alevinfry_example_v0.5.0/permit",
                                package = "alevinQC"),
        quantDir = system.file("extdata/alevinfry_example_v0.5.0/quant",
                               package = "alevinQC"),
        sampleId = "test", outputFile = "tmpp2fry.html",
        outputDir = tempDir, outputFormat = NULL,
        forceOverwrite = FALSE)
    expect_equal(basename(rpt), "tmpp2fry.html")

    ## Try to specify custom barcode lists - currently not supported
    expect_warning(rpt <- alevinFryQCReport(
        mapDir = system.file("extdata/alevinfry_example_v0.5.0/map",
                             package = "alevinQC"),
        permitDir = system.file("extdata/alevinfry_example_v0.5.0/permit",
                                package = "alevinQC"),
        quantDir = system.file("extdata/alevinfry_example_v0.5.0/quant",
                               package = "alevinQC"),
        sampleId = "test", outputFile = "tmppfry3.html",
        outputDir = tempDir, outputFormat = NULL,
        forceOverwrite = TRUE,
        customCBList = list(set1 = c("TCGCGAGGTTCAGACT",
                                     "ATGAGGGAGTAGTGCG"),
                            set2 = c("CGAACATTCTGATACG"))))
    expect_equal(basename(rpt), "tmppfry3.html")
})

test_that("app generation works", {
    app <- alevinFryQCShiny(
        mapDir = system.file("extdata/alevinfry_example_v0.5.0/map",
                             package = "alevinQC"),
        permitDir = system.file("extdata/alevinfry_example_v0.5.0/permit",
                                package = "alevinQC"),
        quantDir = system.file("extdata/alevinfry_example_v0.5.0/quant",
                               package = "alevinQC"),
        sampleId = "test")
    expect_s3_class(app, "shiny.appobj")
})

# Make a summary table for a selection of barcodes in cbtable
# colName should be the name of a logical column in cbtable, indicating
# inclusion/exclusion with respect to the barcode collection
# cbName will be used for display purposes, to identify the barcode collection
.makeSummaryTable <- function(cbtable, colName, cbName = "",
                              countCol = "collapsedFreq", quantmat = NULL) {
    stopifnot(colName %in% colnames(cbtable))
    stopifnot(is.logical(cbtable[[colName]]))

    if (countCol == "nbrMappedUMI") {
        readtype <- " mapped"
    } else {
        readtype <- ""
    }

    df <- list()
    df[[paste0("Number of barcodes", cbName)]] <-
        as.character(sum(cbtable[[colName]], na.rm = TRUE))
    df[[paste0("Number of barcodes with quantification", cbName)]] <-
        as.character(sum(cbtable[[colName]] & !is.na(cbtable$mappingRate),
                         na.rm = TRUE))
    df[[paste0("Fraction", readtype, " reads in barcodes", cbName)]] <-
        as.character(paste0(signif(
            100 * sum(cbtable[[countCol]][cbtable[[colName]]],
                      na.rm = TRUE) /
                sum(cbtable$originalFreq, na.rm = TRUE), 4), "%"))
    df[[paste0("Mean number of", readtype, " reads per cell", cbName)]] <-
        as.character(round(mean(
            cbtable[[countCol]][cbtable[[colName]]],
            na.rm = TRUE)))
    df[[paste0("Median number of", readtype, " reads per cell", cbName)]] <-
        as.character(round(stats::median(
            cbtable[[countCol]][cbtable[[colName]]],
            na.rm = TRUE)))
    df[[paste0("Mean number of detected genes per cell", cbName)]] <-
        as.character(round(mean(
            cbtable$nbrGenesAboveZero[cbtable[[colName]]],
            na.rm = TRUE)))
    df[[paste0("Median number of detected genes per cell", cbName)]] <-
        as.character(round(stats::median(
            cbtable$nbrGenesAboveZero[cbtable[[colName]]],
            na.rm = TRUE)))
    if (!is.null(quantmat)) {
        df[[paste0("Total number of detected genes", cbName)]] <-
            as.character(sum(
                rowSums(quantmat[, colnames(quantmat) %in%
                                     cbtable$CB[cbtable[[colName]]],
                                 drop = FALSE]) > 0))
    }
    df[[paste0("Mean UMI count per cell", cbName)]] <-
        as.character(round(mean(
            cbtable$totalUMICount[cbtable[[colName]]],
            na.rm = TRUE)))
    df[[paste0("Median UMI count per cell", cbName)]] <-
        as.character(round(stats::median(
            cbtable$totalUMICount[cbtable[[colName]]],
            na.rm = TRUE)))
    df <- t(data.frame(df,
                       stringsAsFactors = FALSE,
                       check.names = FALSE))
    df
}


#' Read alevin data required to generate summary report
#'
#' Read all alevin output files required to generate the summary report or shiny
#' app.
#'
#' @param baseDir Path to the output directory from the alevin run (should be
#'   the directory containing the \code{alevin} directory).
#' @param customCBList Named list with custom set(s) of barcodes to provide
#'   summary statistics/plots for, in addition to the whitelists generated by
#'   alevin.
#'
#' @author Charlotte Soneson
#'
#' @export
#'
#' @importFrom utils read.delim
#' @import dplyr
#' @importFrom rjson fromJSON
#' @importFrom tximport tximport
#'
#' @return A list collecting all necessary information for generating the
#'   summary report/shiny app.
#'
#' @examples
#' alevin <- readAlevinQC(system.file("extdata/alevin_example_v0.14",
#'                        package = "alevinQC"))
#'
readAlevinQC <- function(baseDir, customCBList = list()) {
    if (!is.list(customCBList)) {
        stop("'customCBList' must be a (possibly empty) list")
    }
    if (length(customCBList) > 0 && (any(is.null(names(customCBList))) ||
                                     any(names(customCBList) == ""))) {
        stop("'customCBList' must be a named list")
    }

    ## Check that all required files are available, stop if not
    infversion <- checkAlevinInputFiles(baseDir)

    ## Depending on the inferred version, read alevin output files
    if (infversion == "pre0.14") {
        ## pre-v0.14
        .readAlevinQC_pre0.14(baseDir = baseDir, customCBList = customCBList)
    } else if (infversion == "v0.14") {
        ## v0.14 or newer, final whitelist inferred from data
        .readAlevinQC_v0.14(baseDir = baseDir, customCBList = customCBList,
                            type = "standard")
    } else if (infversion == "v0.14extwl") {
        ## v0.14 or newer, external whitelist provided
        .readAlevinQC_v0.14(baseDir = baseDir, customCBList = customCBList,
                            type = "extwl")
    } else if (infversion == "v0.14nowl") {
        ## v0.14 or newer, no whitelist.txt file but no external whitelist
        .readAlevinQC_v0.14(baseDir = baseDir, customCBList = customCBList,
                            type = "nowl")
    } else {
        stop("Unidentifiable alevin output")
    }
}

.readAlevinQC_pre0.14 <- function(baseDir, customCBList = list()) {
    ## No longer supported by tximport (version >= 1.17.4)
    # nocov start

    alevinDir <- file.path(baseDir, "alevin")

    ## Raw CB frequencies (assumed to be in descending order)
    rawcbfreq <- utils::read.delim(file.path(alevinDir, "raw_cb_frequency.txt"),
                                   header = FALSE, as.is = TRUE) %>%
        dplyr::rename(CB = V1, originalFreq = V2) %>%
        dplyr::mutate(ranking = seq_len(length(CB)))

    ## First set of whitelisted CBs (quantified)
    filtcbfreq <- utils::read.delim(file.path(alevinDir,
                                              "filtered_cb_frequency.txt"),
                                    header = FALSE, as.is = TRUE) %>%
        dplyr::rename(CB = V1, collapsedFreq = V2)

    ## FeatureDump
    ## dedupRate = nbr deduplicated UMI counts/nbr mapped reads
    ## nbrGenesAboveMean = nbr genes with count > mean gene count
    featuredump <- utils::read.delim(file.path(alevinDir, "featureDump.txt"),
                                     header = FALSE, as.is = TRUE) %>%
        dplyr::rename(CB = V1, mappingRate = V2, duplicationRate = V3,
                      dedupRate = V4, nbrGenesAboveMean = V5)

    ## Mapped UMI
    mappedumi <- utils::read.delim(file.path(alevinDir, "MappedUmi.txt"),
                                   header = FALSE, as.is = TRUE) %>%
        dplyr::rename(CB = V1, nbrMappedUMI = V2)

    ## Final set of whitelisted CBs
    finalwhitelist <- utils::read.delim(file.path(alevinDir, "whitelist.txt"),
                                        header = FALSE, as.is = TRUE)$V1

    ## Quantification
    quantmat <- tximport::tximport(file.path(baseDir, "alevin/quants_mat.gz"),
                                   type = "alevin")$counts
    quants <- data.frame(CB = colnames(quantmat),
                         totalUMICount = colSums(quantmat),
                         nbrGenesAboveZero = colSums(quantmat > 0),
                         stringsAsFactors = FALSE)

    ## Merge information about quantified CBs
    quantbcs <- filtcbfreq %>%
        dplyr::full_join(featuredump, by = "CB") %>%
        dplyr::full_join(mappedumi, by = "CB") %>%
        dplyr::full_join(quants, by = "CB") %>%
        dplyr::mutate(inFinalWhiteList = CB %in% finalwhitelist) %>%
        dplyr::mutate(inFirstWhiteList = TRUE)

    cbtable <- dplyr::full_join(
        rawcbfreq,
        quantbcs
    ) %>% dplyr::mutate(inFirstWhiteList = replace(inFirstWhiteList,
                                                   is.na(inFirstWhiteList),
                                                   FALSE),
                        inFinalWhiteList = replace(inFinalWhiteList,
                                                   is.na(inFinalWhiteList),
                                                   FALSE))

    ## Add information from custom barcode sets
    customCBsummary <- list()
    for (i in seq_along(customCBList)) {
        nm <- paste0("customCB__", names(customCBList)[i])
        cbtable[[nm]] <- cbtable$CB %in% customCBList[[i]]
        efr <- signif(100 * sum(cbtable[[nm]])/length(customCBList[[i]]), 4)
        qfr <- signif(100 * sum(cbtable[[nm]] & !is.na(cbtable$mappingRate))/
                          length(customCBList[[i]]), 4)
        message(efr, "% of barcodes in custom barcode set ",
                names(customCBList)[i], " were found in the data set (",
                qfr, "% were quantified)")
        customCBsummary[[nm]] <- .makeSummaryTable(
            cbtable = cbtable,
            colName = nm,
            cbName = paste0(" (", gsub("customCB__", "", nm), ")"),
            countCol = "collapsedFreq",
            quantmat = quantmat)
    }

    ## Meta information and command information
    metainfo <- rjson::fromJSON(file = file.path(baseDir,
                                                 "aux_info/meta_info.json"))
    cmdinfo <- rjson::fromJSON(file = file.path(baseDir, "cmd_info.json"))

    ## Create "version info" table
    versiontable <- t(data.frame(
        `Start time` = metainfo$start_time,
        `Salmon version` = metainfo$salmon_version,
        `Index` = cmdinfo$index,
        `R1file` = paste(cmdinfo$mates1,
                         collapse = ", "),
        `R2file` = paste(cmdinfo$mates2,
                         collapse = ", "),
        `tgMap` = cmdinfo$tgMap,
        `Library type` = metainfo$library_types,
        stringsAsFactors = FALSE,
        check.names = FALSE
    ))

    ## Create summary tables
    summarytable_full <- t(data.frame(
        `Total number of processed reads` =
            as.character(metainfo$num_processed),
        `Number of reads with valid cell barcode (no Ns)` =
            as.character(round(sum(rawcbfreq$originalFreq, na.rm = TRUE))),
        `Number of mapped reads` = metainfo$num_mapped,
        `Percent mapped` = metainfo$percent_mapped,
        `Total number of observed cell barcodes` =
            as.character(length(unique(cbtable$CB))),
        stringsAsFactors = FALSE,
        check.names = FALSE
    ))

    summarytable_initialwl <- .makeSummaryTable(
        cbtable = cbtable,
        colName = "inFirstWhiteList",
        cbName = " (initial whitelist)",
        countCol = "collapsedFreq",
        quantmat = quantmat
    )

    summarytable_finalwl <- .makeSummaryTable(
        cbtable = cbtable,
        colName = "inFinalWhiteList",
        cbName = " (final whitelist)",
        countCol = "collapsedFreq",
        quantmat = quantmat
    )

    ## Return
    list(cbTable = cbtable, versionTable = versiontable,
         summaryTables = c(list(fullDataset = summarytable_full,
                                initialWhitelist = summarytable_initialwl,
                                finalWhitelist = summarytable_finalwl),
                           customCBsummary),
         type = "pre0.14"
    )
    # nocov end
}

.readAlevinQC_v0.14 <- function(baseDir, customCBList = list(), type = "standard") {

    alevinDir <- file.path(baseDir, "alevin")

    ## Raw CB frequencies (assumed to be in descending order)
    rawcbfreq <- utils::read.delim(file.path(alevinDir, "raw_cb_frequency.txt"),
                                   header = FALSE, as.is = TRUE) %>%
        dplyr::rename(CB = V1, originalFreq = V2) %>%
        dplyr::mutate(ranking = seq_len(length(CB)))
    if (!all(diff(rawcbfreq$originalFreq) <= 0)) {
        warning("The raw CB frequencies are not sorted in decreasing order")
    }

    ## FeatureDump
    featuredump <- try({
        utils::read.delim(file.path(alevinDir, "featureDump.txt"),
                          header = TRUE, as.is = TRUE, sep = "\t")
    }, silent = TRUE)
    if (is(featuredump, "try-error")) {
        if (grepl("more columns than column names", featuredump)) {
            warning("The 'featureDump.txt' file could not be cleanly read. ",
                    "Please check results carefully (e.g. by reading the ",
                    "input manually and checking the 'cbTable' output.")
            ## In alevin 1.0.0 (fixed in 1.1.0), with certain flags,
            ## the ArborescenceCount column could be split over multiple columns
            tmpcn <- unlist(utils::read.delim(
                file.path(alevinDir, "featureDump.txt"),
                header = FALSE, as.is = TRUE, sep = "\t", nrows = 1
            ))
            tmpdt <- utils::read.delim(
                file.path(alevinDir, "featureDump.txt"),
                header = FALSE, as.is = TRUE, sep = "\t", skip = 1
            )
            if (length(tmpcn) != ncol(tmpdt)) {
                tmpdt <- tmpdt[, seq_len(length(tmpcn)), drop = FALSE]
            }
            colnames(tmpdt) <- tmpcn
            featuredump <- tmpdt
        } else {
            stop("The 'featureDump.txt' file could not be read.")
        }
    }
    featuredump <- featuredump %>%
        dplyr::rename(mappingRate = MappingRate,
                      collapsedFreq = CorrectedReads,
                      dedupRate = DedupRate,
                      nbrGenesAboveMean = NumGenesOverMean,
                      nbrMappedUMI = MappedReads,
                      totalUMICount = DeduplicatedReads,
                      nbrGenesAboveZero = NumGenesExpressed)

    if (type == "standard") {
        ## Final set of whitelisted CBs
        finalwhitelist <- utils::read.delim(
            file.path(alevinDir, "whitelist.txt"),
            header = FALSE, as.is = TRUE)$V1
    }

    ## Meta information and command information
    metainfo <- rjson::fromJSON(file = file.path(baseDir,
                                                 "aux_info/meta_info.json"))
    cmdinfo <- rjson::fromJSON(file = file.path(baseDir, "cmd_info.json"))
    alevinmetainfo <- rjson::fromJSON(
        file = file.path(baseDir, "aux_info/alevin_meta_info.json"))

    ## Merge information about quantified CBs
    cbtable <- dplyr::full_join(
        rawcbfreq,
        featuredump,
        by = "CB"
    )
    if (type == "standard") {
        ## we have a whitelist.txt file representing the final whitelist
        cbtable <- cbtable %>%
            dplyr::mutate(inFinalWhiteList = CB %in% finalwhitelist) %>%
            dplyr::mutate(
                inFirstWhiteList = ranking <= alevinmetainfo$initial_whitelist
            )
        ## Check if there is any barcode that is not in the first whitelist,
        ## but which has an original ranking lower than any barcode that is
        ## in the first whitelist, and remove it.
        toremove <-
            !cbtable$inFirstWhiteList &
            cbtable$ranking <= max(cbtable$ranking[cbtable$inFirstWhiteList])
        if (any(toremove)) {
            warning("Excluding ", sum(toremove), " unquantified barcode",
                    ifelse(sum(toremove) > 1, "s", ""),
                    " with higher original frequency than barcodes ",
                    "included in the first whitelist: ",
                    paste0(cbtable$CB[toremove], collapse = ", "))
            cbtable <- cbtable[!toremove, ]
        }
    } else if (type == "nowl") {
        ## we have an indication of the size of the initial whitelist, but
        ## no final whitelist (and no whitelist.txt file).
        ## the final number of considered CBs should be those in the initial
        ## whitelist
        cbtable <- cbtable %>%
            dplyr::mutate(
                inFirstWhiteList = ranking <= alevinmetainfo$initial_whitelist
            ) %>%
            dplyr::mutate(inFinalWhiteList = inFirstWhiteList)
    } else if (type == "extwl") {
        ## the CBs included in the featureDump file are the ones included
        ## in the external whitelist. There is no initial whitelisting step
        cbtable <- cbtable %>%
            dplyr::mutate(inFirstWhiteList = CB %in% featuredump$CB) %>%
            dplyr::mutate(inFinalWhiteList = inFirstWhiteList)
    }

    ## Add information from custom barcode sets
    customCBsummary <- list()
    for (i in seq_along(customCBList)) {
        nm <- paste0("customCB__", names(customCBList)[i])
        cbtable[[nm]] <- cbtable$CB %in% customCBList[[i]]
        efr <- signif(100 * sum(cbtable[[nm]])/length(customCBList[[i]]), 4)
        qfr <- signif(100 * sum(cbtable[[nm]] & !is.na(cbtable$mappingRate))/
                          length(customCBList[[i]]), 4)
        message(efr, "% of barcodes in custom barcode set ",
                names(customCBList)[i], " were found in the data set (",
                qfr, "% were quantified)")
        customCBsummary[[nm]] <- .makeSummaryTable(
            cbtable = cbtable,
            colName = nm,
            cbName = paste0(" (", gsub("customCB__", "", nm), ")"),
            countCol = "collapsedFreq",
            quantmat = NULL)
    }

    ## Create "version info" table
    versiontable <- t(data.frame(
        `Start time` = metainfo$start_time,
        `Salmon version` = metainfo$salmon_version,
        `Index` = cmdinfo$index,
        `R1file` = paste(cmdinfo$mates1,
                         collapse = ", "),
        `R2file` = paste(cmdinfo$mates2,
                         collapse = ", "),
        `tgMap` = cmdinfo$tgMap,
        `Library type` = metainfo$library_types,
        stringsAsFactors = FALSE,
        check.names = FALSE
    ))
    if (type == "extwl") {
        versiontable <- rbind(
            versiontable,
            t(data.frame(
                `External whitelist` = cmdinfo$whitelist,
                stringsAsFactors = FALSE,
                check.names = FALSE
            ))
        )
    }
    if ("expectCells" %in% names(cmdinfo)) {
        versiontable <- rbind(
            versiontable,
            t(data.frame(
                `Expected number of cells` = cmdinfo$expectCells,
                stringsAsFactors = FALSE,
                check.names = FALSE
            ))
        )
    }
    if ("forceCells" %in% names(cmdinfo)) {
        versiontable <- rbind(
            versiontable,
            t(data.frame(
                `Forced number of cells` = cmdinfo$forceCells,
                stringsAsFactors = FALSE,
                check.names = FALSE
            ))
        )
    }

    ## Create summary tables
    summarytable_full <- t(data.frame(
        `Total number of processed reads` =
            as.character(alevinmetainfo$total_reads),
        `Number of reads with Ns` =
            as.character(alevinmetainfo$reads_with_N),
        `Number of reads with valid cell barcode (no Ns)` =
            as.character(round(sum(rawcbfreq$originalFreq, na.rm = TRUE))),
        `Number of mapped reads` = alevinmetainfo$reads_in_eqclasses,
        `Percent mapped (of all reads)` =
            paste0(signif(as.numeric(alevinmetainfo$mapping_rate), 4), "%"),
        `Number of noisy CB reads` =
            as.character(alevinmetainfo$noisy_cb_reads),
        `Number of noisy UMI reads` =
            as.character(alevinmetainfo$noisy_umi_reads),
        `Total number of observed cell barcodes` =
            as.character(length(unique(cbtable$CB))),
        stringsAsFactors = FALSE,
        check.names = FALSE
    ))

    ## If used_reads is reported, add actual mapping rate
    if (!is.null(alevinmetainfo$used_reads)) {
        summarytable_full <- rbind(
            summarytable_full,
            t(data.frame(
                `Number of used reads` = alevinmetainfo$used_reads,
                `Percent mapped (of used reads)` =
                    paste0(signif(100 * alevinmetainfo$reads_in_eqclasses /
                    alevinmetainfo$used_reads, 4), "%"),
                stringsAsFactors = FALSE,
                check.names = FALSE
            ))
        )
    }

    summarytable_initialwl <- .makeSummaryTable(
        cbtable = cbtable,
        colName = "inFirstWhiteList",
        cbName = " (initial whitelist)",
        countCol = "collapsedFreq",
        quantmat = NULL
    )

    summarytable_finalwl <- .makeSummaryTable(
        cbtable = cbtable,
        colName = "inFinalWhiteList",
        cbName = " (final whitelist)",
        countCol = "collapsedFreq",
        quantmat = NULL
    )

    ## Return
    list(cbTable = cbtable, versionTable = versiontable,
         summaryTables = c(list(fullDataset = summarytable_full,
                                initialWhitelist = summarytable_initialwl,
                                finalWhitelist = summarytable_finalwl),
                           customCBsummary),
         type = type
    )
}

test_that("checking for input files works", {
    ## All required files available - check should pass
    expect_equal(checkAlevinInputFiles(system.file("extdata/alevin_example_v0.14",
                                                   package = "alevinQC")),
                 "v0.14")

    ## Remove one file - check should fail
    tmp <- tempdir()
    file.copy(from = system.file("extdata/alevin_example_v0.14",
                                 package = "alevinQC"),
              to = tmp, overwrite = TRUE, recursive = TRUE)
    file.remove(file.path(tmp, "alevin_example_v0.14/cmd_info.json"))
    expect_error(checkAlevinInputFiles(file.path(tmp, "alevin_example_v0.14")))

    ## Remove whitelist file - should be read differently
    tmp <- tempdir()
    file.copy(from = system.file("extdata/alevin_example_v0.14",
                                 package = "alevinQC"),
              to = tmp, overwrite = TRUE, recursive = TRUE)
    file.remove(file.path(tmp, "alevin_example_v0.14/alevin/whitelist.txt"))
    expect_equal(checkAlevinInputFiles(file.path(tmp, "alevin_example_v0.14")), "v0.14nowl")

    ## Add whitelist entry to json - should be read differently
    a <- rjson::fromJSON(file = file.path(tmp, "alevin_example_v0.14/cmd_info.json"))
    a$whitelist = "whitelist.txt"
    write(rjson::toJSON(a), file = file.path(tmp, "alevin_example_v0.14/cmd_info.json"))
    expect_equal(checkAlevinInputFiles(file.path(tmp, "alevin_example_v0.14")), "v0.14extwl")
})

## Read provided example input files for tests of file reading/plotting
expect_message(
    expect_message(alevin <- readAlevinQC(
        system.file("extdata/alevin_example_v0.14",
                    package = "alevinQC"),
        customCBList = list(set1 = c("TCGCGAGGTTCAGACT",
                                     "ATGAGGGAGTAGTGCG"),
                            set2 = c("CGAACATTCTGATACG"))),
        "100% of barcodes in custom barcode set set1 were found in the data set"),
    "100% of barcodes in custom barcode set set2 were found in the data set")

test_that("reading input files works", {
    expect_length(alevin, 4)
    expect_type(alevin, "list")
    expect_named(alevin, c("cbTable", "versionTable", "summaryTables", "type"))
    expect_equal(alevin$type, "standard")

    expect_equal(nrow(alevin$cbTable), 188613)
    expect_equal(sum(alevin$cbTable$inFirstWhiteList), 100)
    expect_equal(sum(!is.na(alevin$cbTable$mappingRate)), 298)
    expect_equal(sum(alevin$cbTable$inFinalWhiteList), 95)

    expect_equal(
        alevin$summaryTables$customCB__set2["Number of barcodes (set2)", 1],
        "1", ignore_attr = TRUE
    )
    expect_equal(
        alevin$summaryTables$customCB__set2["Mean number of reads per cell (set2)", 1],
        "106072", ignore_attr = TRUE
    )

    expect_equal(sum(alevin$cbTable$collapsedFreq[alevin$cbTable$customCB__set1]),
                 95259 + 108173)
    expect_equal(alevin$cbTable$collapsedFreq[alevin$cbTable$customCB__set2],
                 106072)
    expect_error(.makeSummaryTable(alevin$cbTable, colName = "collapsedFreq"))
    expect_error(.makeSummaryTable(alevin$cbTable, colName = "missingCol"))
})

test_that("plots are generated", {
    expect_true(ggplot2::is_ggplot(plotAlevinKneeRaw(alevin$cbTable)))
    cbTableTmp <- alevin$cbTable
    cbTableTmp$inFirstWhiteList[1] <- FALSE
    expect_true(ggplot2::is_ggplot(plotAlevinKneeRaw(cbTableTmp)))
    expect_true(ggplot2::is_ggplot(plotAlevinBarcodeCollapse(alevin$cbTable)))
    expect_s3_class(plotAlevinQuantPairs(alevin$cbTable), "ggmatrix")
    expect_true(ggplot2::is_ggplot(plotAlevinKneeNbrGenes(alevin$cbTable)))
    expect_true(ggplot2::is_ggplot(plotAlevinHistogram(alevin$cbTable)))

    expect_error(plotAlevinQuantPairs(alevin$cbTable,
                                      colName = "nbrGenesAboveMean"))
    expect_error(plotAlevinQuant(alevin$cbTable,
                                 colName = "nbrGenesAboveMean"))
    expect_error(plotAlevinHistogram(alevin$cbTable,
                                     colName = "missing"))
    expect_error(plotAlevinHistogram(alevin$cbTable,
                                     colName = "CB"))
})

## Read provided example input files for tests of file reading/plotting, after removing whitelist
tmp <- tempdir()
file.copy(from = system.file("extdata/alevin_example_v0.14",
                             package = "alevinQC"),
          to = tmp, overwrite = TRUE, recursive = TRUE)
file.remove(file.path(tmp, "alevin_example_v0.14/alevin/whitelist.txt"))
expect_message(
    expect_message(alevin <- readAlevinQC(
        file.path(tmp, "alevin_example_v0.14"),
        customCBList = list(set1 = c("TCGCGAGGTTCAGACT",
                                     "ATGAGGGAGTAGTGCG"),
                            set2 = c("CGAACATTCTGATACG"))),
        "100% of barcodes in custom barcode set set1 were found in the data set"),
    "100% of barcodes in custom barcode set set2 were found in the data set")

test_that("reading input files works", {
    expect_length(alevin, 4)
    expect_type(alevin, "list")
    expect_named(alevin, c("cbTable", "versionTable", "summaryTables", "type"))
    expect_equal(alevin$type, "nowl")

    expect_equal(nrow(alevin$cbTable), 188613)
    expect_equal(sum(alevin$cbTable$inFirstWhiteList), 100)
    expect_equal(sum(!is.na(alevin$cbTable$mappingRate)), 298)
    expect_equal(sum(alevin$cbTable$inFinalWhiteList), 100)

    expect_equal(
        alevin$summaryTables$customCB__set2["Number of barcodes (set2)", 1],
        "1", ignore_attr = TRUE
    )
    expect_equal(
        alevin$summaryTables$customCB__set2["Mean number of reads per cell (set2)", 1],
        "106072", ignore_attr = TRUE
    )

    expect_equal(sum(alevin$cbTable$collapsedFreq[alevin$cbTable$customCB__set1]),
                 95259 + 108173)
    expect_equal(alevin$cbTable$collapsedFreq[alevin$cbTable$customCB__set2],
                 106072)
    expect_error(.makeSummaryTable(alevin$cbTable, colName = "collapsedFreq"))
    expect_error(.makeSummaryTable(alevin$cbTable, colName = "missingCol"))
})


tempDir <- tempdir()
if (file.exists(file.path(tempDir, "tmp.Rmd"))) {
    file.remove(file.path(tempDir, "tmp.Rmd"))
}
if (file.exists(file.path(tempDir, "tmp.html"))) {
    file.remove(file.path(tempDir, "tmp.html"))
}
if (file.exists(file.path(tempDir, "tmp2.html"))) {
    file.remove(file.path(tempDir, "tmp2.html"))
}
if (file.exists(file.path(tempDir, "tmp.pdf"))) {
    file.remove(file.path(tempDir, "tmp.pdf"))
}
if (file.exists(file.path(tempDir, "tmp2.pdf"))) {
    file.remove(file.path(tempDir, "tmp2.pdf"))
}

test_that("input arguments are processed correctly", {
    ## outputFormat
    expect_error(alevinQCReport(baseDir = system.file("extdata/alevin_example_v0.14",
                                                      package = "alevinQC"),
                                outputFormat = "html", outputFile = "tmp.html",
                                outputDir = tempDir, sampleId = "test"))
    expect_error(alevinQCReport(baseDir = system.file("extdata/alevin_example_v0.14",
                                                      package = "alevinQC"),
                                outputFormat = "html_document", outputFile = "tmp.pdf",
                                outputDir = tempDir, sampleId = "test"))
    expect_error(alevinQCReport(baseDir = system.file("extdata/alevin_example_v0.14",
                                                      package = "alevinQC"),
                                outputFormat = "BiocStyle::html_document",
                                outputFile = "tmp.pdf",
                                outputDir = tempDir, sampleId = "test"))
    expect_error(alevinQCReport(baseDir = system.file("extdata/alevin_example_v0.14",
                                                      package = "alevinQC"),
                                outputFormat = "BiocStyle::pdf_document",
                                outputFile = "tmp.html",
                                outputDir = tempDir, sampleId = "test"))
    expect_error(alevinQCReport(baseDir = system.file("extdata/alevin_example_v0.14",
                                                      package = "alevinQC"),
                                outputFormat = "html_document", outputFile = "tmp.html",
                                outputDir = tempDir, sampleId = c("s1", "s2")))
    expect_error(alevinQCReport(baseDir = system.file("extdata/alevin_example_v0.14",
                                                      package = "alevinQC"),
                                outputFormat = "html_document", outputFile = "tmp.html",
                                outputDir = tempDir, sampleId = "s1",
                                customCBList = c("A", "B")))
    expect_error(alevinQCReport(baseDir = system.file("extdata/alevin_example_v0.14",
                                                      package = "alevinQC"),
                                outputFormat = "html_document", outputFile = "tmp.html",
                                outputDir = tempDir, sampleId = "s1",
                                customCBList = list(c("A", "B"), c("C", "D"))))
})

test_that("report generation works", {
    rpt <- alevinQCReport(baseDir = system.file("extdata/alevin_example_v0.14",
                                                package = "alevinQC"),
                          sampleId = "test", outputFile = "tmp.html",
                          outputDir = tempDir, outputFormat = NULL,
                          forceOverwrite = FALSE,
                          customCBList = list(set1 = c("TCGCGAGGTTCAGACT",
                                                       "ATGAGGGAGTAGTGCG"),
                                              set2 = c("CGAACATTCTGATACG")))
    expect_equal(basename(rpt), "tmp.html")
    expect_error(alevinQCReport(baseDir = system.file("extdata/alevin_example_v0.14",
                                                      package = "alevinQC"),
                                sampleId = "test", outputFile = "tmp.html",
                                outputDir = tempDir, outputFormat = NULL,
                                forceOverwrite = FALSE,
                                customCBList = list(set1 = c("TCGCGAGGTTCAGACT",
                                                             "ATGAGGGAGTAGTGCG"),
                                                    set2 = c("CGAACATTCTGATACG"))))
    file.copy(system.file("extdata/alevin_report_template.Rmd",
                          package = "alevinQC"),
              file.path(tempDir, "tmp.Rmd"))
    expect_warning(
        expect_error(
            alevinQCReport(baseDir = system.file("extdata/alevin_example_v0.14",
                                                 package = "alevinQC"),
                           sampleId = "test", outputFile = "tmp.html",
                           outputDir = tempDir, outputFormat = NULL,
                           forceOverwrite = TRUE,
                           customCBList = list(set1 = c("TCGCGAGGTTCAGACT",
                                                        "ATGAGGGAGTAGTGCG"),
                                               set2 = c("CGAACATTCTGATACG")))
        )
    )
    rpt <- alevinQCReport(baseDir = system.file("extdata/alevin_example_v0.14",
                                                package = "alevinQC"),
                          sampleId = "test", outputFile = "tmp2.html",
                          outputDir = tempDir, outputFormat = NULL,
                          forceOverwrite = FALSE)
    expect_equal(basename(rpt), "tmp2.html")
    expect_warning(
        rpt <- alevinQCReport(baseDir = system.file("extdata/alevin_example_v0.14",
                                                    package = "alevinQC"),
                              sampleId = "test", outputFile = "tmp2.html",
                              outputDir = tempDir,
                              outputFormat = "BiocStyle::html_document",
                              forceOverwrite = TRUE))
    expect_equal(basename(rpt), "tmp2.html")
    ## BiocStyle::pdf_document generates lots of warnings related to fancyhdr
    ## Also an error 'Command \@raggedtwoe@everyselectfont undefined'
    # wns <- capture_warnings(
    #     rpt <- alevinQCReport(baseDir = system.file("extdata/alevin_example_v0.14",
    #                                                 package = "alevinQC"),
    #                           sampleId = "test", outputFile = "tmp2.pdf",
    #                           outputDir = tempDir,
    #                           outputFormat = "BiocStyle::pdf_document",
    #                           forceOverwrite = TRUE))
    # expect_equal(basename(rpt), "tmp2.pdf")
    # expect_true(length(wns) > 1)
})

test_that("app generation works", {
    app <- alevinQCShiny(baseDir = system.file("extdata/alevin_example_v0.14",
                                               package = "alevinQC"),
                         sampleId = "test")
    expect_s3_class(app, "shiny.appobj")
    expect_message(
        expect_message(
            app <- alevinQCShiny(baseDir = system.file("extdata/alevin_example_v0.14",
                                                       package = "alevinQC"),
                                 sampleId = "test",
                                 customCBList = list(set1 = c("TCGCGAGGTTCAGACT",
                                                              "ATGAGGGAGTAGTGCG"),
                                                     set2 = c("CGAACATTCTGATACG"))),
            "100% of barcodes in custom barcode set set1 were found in the data set"),
        "100% of barcodes in custom barcode set set2 were found in the data set")
    expect_s3_class(app, "shiny.appobj")
})

test_that("checking for input files works", {
    ## All required files available - check should pass
    expect_equal(checkAlevinInputFiles(system.file("extdata/alevin_example_pre0.14",
                                                   package = "alevinQC")),
                 "pre0.14")

    ## Remove one file - check should fail
    tmp <- tempdir()
    file.copy(from = system.file("extdata/alevin_example_pre0.14",
                                 package = "alevinQC"),
              to = tmp, overwrite = TRUE, recursive = TRUE)
    file.remove(file.path(tmp, "alevin_example_pre0.14/cmd_info.json"))
    expect_error(checkAlevinInputFiles(file.path(tmp, "alevin_example_pre0.14")))
})

## Read provided example input files for tests of file reading/plotting
## alevin < 0.14 not supported by tximport >= 1.17.4
expect_error(alevin <- readAlevinQC(
    system.file("extdata/alevin_example_pre0.14",
                package = "alevinQC"),
    customCBList = list(set1 = c("TCGCGAGGTTCAGACT",
                                 "ATGAGGGAGTAGTGCG"),
                        set2 = c("CGAACATTCTGATACG"))))

# test_that("reading input files works", {
#     expect_length(alevin, 4)
#     expect_is(alevin, "list")
#     expect_named(alevin, c("cbTable", "versionTable", "summaryTables", "type"))
#     expect_equal(alevin$type, "pre0.14")
#
#     expect_equal(nrow(alevin$cbTable), 188613)
#     expect_equal(sum(alevin$cbTable$inFirstWhiteList), 299)
#     expect_equal(sum(!is.na(alevin$cbTable$mappingRate)), 298)
#     expect_equal(sum(alevin$cbTable$inFinalWhiteList), 98)
#
#     expect_equivalent(alevin$summaryTables$customCB__set2["Number of barcodes (set2)", 1], "1")
#     expect_equivalent(alevin$summaryTables$customCB__set2["Mean number of reads per cell (set2)", 1], "106072")
#
#     expect_equal(sum(alevin$cbTable$collapsedFreq[alevin$cbTable$customCB__set1]),
#                  95259 + 108173)
#     expect_equal(alevin$cbTable$collapsedFreq[alevin$cbTable$customCB__set2],
#                  106072)
#     expect_error(.makeSummaryTable(alevin$cbTable, colName = "collapsedFreq"))
#     expect_error(.makeSummaryTable(alevin$cbTable, colName = "missingCol"))
# })
#
# test_that("plots are generated", {
#     expect_true(ggplot2::is_ggplot(plotAlevinKneeRaw(alevin$cbTable)))
#     expect_true(ggplot2::is_ggplot(plotAlevinBarcodeCollapse(alevin$cbTable)))
#     expect_is(plotAlevinQuantPairs(alevin$cbTable), "ggmatrix")
#     expect_true(ggplot2::is_ggplot(plotAlevinKneeNbrGenes(alevin$cbTable)))
#     expect_true(ggplot2::is_ggplot(plotAlevinHistogram(alevin$cbTable)))
# })
#
# tempDir <- tempdir()
# if (file.exists(file.path(tempDir, "tmp.Rmd"))) {
#     file.remove(file.path(tempDir, "tmp.Rmd"))
# }
# if (file.exists(file.path(tempDir, "tmp.html"))) {
#     file.remove(file.path(tempDir, "tmp.html"))
# }
# if (file.exists(file.path(tempDir, "tmp2.html"))) {
#     file.remove(file.path(tempDir, "tmp2.html"))
# }
# if (file.exists(file.path(tempDir, "tmp.pdf"))) {
#     file.remove(file.path(tempDir, "tmp.pdf"))
# }
#
# test_that("input arguments are processed correctly", {
#     ## outputFormat
#     expect_error(alevinQCReport(baseDir = system.file("extdata/alevin_example_pre0.14",
#                                                       package = "alevinQC"),
#                                 outputFormat = "html", outputFile = "tmp.html",
#                                 outputDir = tempDir, sampleId = "test"))
#     expect_error(alevinQCReport(baseDir = system.file("extdata/alevin_example_pre0.14",
#                                                       package = "alevinQC"),
#                                 outputFormat = "html_document", outputFile = "tmp.pdf",
#                                 outputDir = tempDir, sampleId = "test"))
#     expect_error(alevinQCReport(baseDir = system.file("extdata/alevin_example_pre0.14",
#                                                       package = "alevinQC"),
#                                 outputFormat = "html_document", outputFile = "tmp.html",
#                                 outputDir = tempDir, sampleId = c("s1", "s2")))
#     expect_error(alevinQCReport(baseDir = system.file("extdata/alevin_example_pre0.14",
#                                                       package = "alevinQC"),
#                                 outputFormat = "html_document", outputFile = "tmp.html",
#                                 outputDir = tempDir, sampleId = "s1",
#                                 customCBList = c("A", "B")))
#     expect_error(alevinQCReport(baseDir = system.file("extdata/alevin_example_pre0.14",
#                                                       package = "alevinQC"),
#                                 outputFormat = "html_document", outputFile = "tmp.html",
#                                 outputDir = tempDir, sampleId = "s1",
#                                 customCBList = list(c("A", "B"), c("C", "D"))))
# })
#
# test_that("report generation works", {
#     rpt <- alevinQCReport(baseDir = system.file("extdata/alevin_example_pre0.14",
#                                                 package = "alevinQC"),
#                           sampleId = "test", outputFile = "tmp.html",
#                           outputDir = tempDir, outputFormat = NULL,
#                           forceOverwrite = FALSE,
#                           customCBList = list(set1 = c("TCGCGAGGTTCAGACT",
#                                                        "ATGAGGGAGTAGTGCG"),
#                                               set2 = c("CGAACATTCTGATACG")))
#     expect_equal(basename(rpt), "tmp.html")
#     expect_error(alevinQCReport(baseDir = system.file("extdata/alevin_example_pre0.14",
#                                                       package = "alevinQC"),
#                                 sampleId = "test", outputFile = "tmp.html",
#                                 outputDir = tempDir, outputFormat = NULL,
#                                 forceOverwrite = FALSE,
#                                 customCBList = list(set1 = c("TCGCGAGGTTCAGACT",
#                                                              "ATGAGGGAGTAGTGCG"),
#                                                     set2 = c("CGAACATTCTGATACG"))))
#     file.copy(system.file("extdata/alevin_report_template.Rmd",
#                           package = "alevinQC"),
#               file.path(tempDir, "tmp.Rmd"))
#     expect_error(alevinQCReport(baseDir = system.file("extdata/alevin_example_pre0.14",
#                                                       package = "alevinQC"),
#                                 sampleId = "test", outputFile = "tmp.html",
#                                 outputDir = tempDir, outputFormat = NULL,
#                                 forceOverwrite = TRUE,
#                                 customCBList = list(set1 = c("TCGCGAGGTTCAGACT",
#                                                              "ATGAGGGAGTAGTGCG"),
#                                                     set2 = c("CGAACATTCTGATACG"))))
#     rpt <- alevinQCReport(baseDir = system.file("extdata/alevin_example_v0.14",
#                                                 package = "alevinQC"),
#                           sampleId = "test", outputFile = "tmp2.html",
#                           outputDir = tempDir, outputFormat = NULL,
#                           forceOverwrite = FALSE)
#     expect_equal(basename(rpt), "tmp2.html")
# })
#
# test_that("app generation works", {
#     app <- alevinQCShiny(baseDir = system.file("extdata/alevin_example_pre0.14",
#                                                package = "alevinQC"),
#                          sampleId = "test")
#     expect_s3_class(app, "shiny.appobj")
#     app <- alevinQCShiny(baseDir = system.file("extdata/alevin_example_pre0.14",
#                                                package = "alevinQC"),
#                          sampleId = "test",
#                          customCBList = list(set1 = c("TCGCGAGGTTCAGACT",
#                                                       "ATGAGGGAGTAGTGCG"),
#                                              set2 = c("CGAACATTCTGATACG")))
#     expect_s3_class(app, "shiny.appobj")
# })

#' Knee plot of the number of detected genes per cell
#'
#' Plot the number of detected genes per cell in decreasing order.
#' Only cells contained in the original whitelist are considered.
#'
#' @author Charlotte Soneson
#'
#' @param cbTable \code{data.frame} (such as the \code{cbTable} returned by
#'     \code{readAlevinQC} or \code{readAlevinFryQC}) with the number of
#'     detected genes per cell.
#' @param firstSelColName Character scalar indicating the name of the logical
#'     column in \code{cbTable} that corresponds to the original selection of
#'     barcodes for quantification.
#'
#' @export
#'
#' @importFrom ggplot2 ggplot aes geom_line xlab ylab theme_bw theme
#'     element_text
#' @import dplyr
#' @importFrom rlang .data
#'
#' @return A ggplot object
#'
#' @examples
#' alevin <- readAlevinQC(system.file("extdata/alevin_example_v0.14",
#'                                    package = "alevinQC"))
#' plotAlevinKneeNbrGenes(alevin$cbTable)
#'
plotAlevinKneeNbrGenes <- function(cbTable,
                                   firstSelColName = "inFirstWhiteList") {
    ## Check input arguments
    .assertVector(x = cbTable, type = "data.frame")
    .assertScalar(x = firstSelColName, type = "character",
                  validValues = colnames(cbTable))
    .assertVector(x = cbTable[[firstSelColName]], type = "logical")
    stopifnot(all(c("nbrGenesAboveZero") %in%
                      colnames(cbTable)))

    ggplot2::ggplot(cbTable %>% dplyr::filter(.data[[firstSelColName]]) %>%
                        dplyr::arrange(dplyr::desc(.data$nbrGenesAboveZero)) %>%
                        dplyr::mutate(x = seq_along(.data$nbrGenesAboveZero)),
                    ggplot2::aes(x = .data$x, y = .data$nbrGenesAboveZero)) +
        ggplot2::geom_line(linewidth = 2) +
        ggplot2::xlab("Cell barcode rank") +
        ggplot2::ylab("Number of detected genes") +
        ggplot2::theme_bw() +
        ggplot2::theme(legend.position = "none",
                       axis.title = ggplot2::element_text(size = 12))
}

#' Read alevin-fry data required to generate summary report
#'
#' Read all alevin-fry output files required to generate the
#' summary report or shiny app.
#'
#' @param mapDir Path to the output directory from the \code{salmon alevin}
#'     run (should be the directory containing the \code{alevin} folder).
#' @param permitDir Path to the output directory from the
#'     \code{generate-permit-list} and \code{collate} runs.
#' @param quantDir Path to the output directory from the
#'     \code{alevin-fry quant} run (should be the directory containing the
#'     \code{alevin} folder).
#'
#' @author Charlotte Soneson
#'
#' @export
#'
#' @importFrom utils read.delim
#' @import dplyr
#' @importFrom rjson fromJSON
#'
#' @return A list collecting all necessary information for generating the
#'     summary report/shiny app.
#'
#' @examples
#' alevinfry <- readAlevinFryQC(
#'     mapDir = system.file("extdata/alevinfry_example_v0.5.0/map",
#'                          package = "alevinQC"),
#'     permitDir = system.file("extdata/alevinfry_example_v0.5.0/permit",
#'                             package = "alevinQC"),
#'     quantDir = system.file("extdata/alevinfry_example_v0.5.0/quant",
#'                            package = "alevinQC"))
#'
readAlevinFryQC <- function(mapDir, permitDir, quantDir) {
    ## Check that all required files are available, stop if not
    infversion <- checkAlevinFryInputFiles(mapDir = mapDir,
                                           permitDir = permitDir,
                                           quantDir = quantDir)

    ## Depending on the inferred version, read alevin output files
    if (infversion == "v0.4.3") {
        ## v0.4.3 or newer
        .readAlevinFryQC_v0.4.3(mapDir = mapDir, permitDir = permitDir,
                                quantDir = quantDir)
    } else if (infversion == "v0.5.0") {
        ## v0.5.0 or newer
        .readAlevinFryQC_v0.5.0(mapDir = mapDir, permitDir = permitDir,
                                quantDir = quantDir)
    } else if (infversion == "piscem_v0.6.0") {
        ## v0.5.0 or newer
        .readAlevinFryQC_piscemv0.6.0(mapDir = mapDir, permitDir = permitDir,
                                      quantDir = quantDir)
    } else {
        stop("Unidentifiable alevin-fry output")
    }
}

.readAlevinFryQC_v0.4.3 <- function(mapDir, permitDir, quantDir) {

    ## Raw CB frequencies (in descending order)
    rawcbfreq <- utils::read.delim(file.path(permitDir, "all_freq.tsv"),
                                   header = FALSE, as.is = TRUE) %>%
        dplyr::rename(CB = V1, originalFreq = V2) %>%
        dplyr::arrange(dplyr::desc(originalFreq)) %>%
        dplyr::mutate(ranking = seq_len(length(CB)))
    if (!all(diff(rawcbfreq$originalFreq) <= 0)) {
        warning("The raw CB frequencies are not sorted in decreasing order")
    }

    ## FeatureDump
    featuredump <- utils::read.delim(file.path(quantDir, "featureDump.txt"),
                          header = TRUE, as.is = TRUE, sep = "\t")
    featuredump <- featuredump %>%
        dplyr::rename(mappingRate = MappingRate,
                      collapsedFreq = CorrectedReads,
                      dedupRate = DedupRate,
                      nbrGenesAboveMean = NumGenesOverMean,
                      nbrMappedUMI = MappedReads,
                      totalUMICount = DeduplicatedReads,
                      nbrGenesAboveZero = NumGenesExpressed)

    permitlist <- utils::read.delim(file.path(quantDir, "alevin",
                                              "quants_mat_rows.txt"),
                                    header = FALSE, as.is = TRUE) %>%
        dplyr::pull(V1)

    ## Meta information and command information
    metainfo <- rjson::fromJSON(file = file.path(mapDir,
                                                 "aux_info/meta_info.json"))
    cmdinfo <- rjson::fromJSON(file = file.path(mapDir, "cmd_info.json"))
    quantinfo <- rjson::fromJSON(file = file.path(quantDir, "quant.json"))

    ## Merge information about quantified CBs
    cbtable <- dplyr::full_join(
        rawcbfreq,
        featuredump,
        by = "CB"
    )

    cbtable <- cbtable %>%
        dplyr::mutate(inPermitList = CB %in% permitlist)

    ## Check if there is any barcode that is not in the permitlist,
    ## but which has an original ranking lower than any barcode that is
    ## in the permitlist, and remove it.
    # toremove <-
    #     !cbtable$inPermitList &
    #     cbtable$ranking <= max(cbtable$ranking[cbtable$inPermitList])
    # if (any(toremove)) {
    #     warning("Excluding ", sum(toremove), " unquantified barcode",
    #             ifelse(sum(toremove) > 1, "s", ""),
    #             " with higher original frequency than barcodes ",
    #             "included in the permitlist: ",
    #             paste0(cbtable$CB[toremove], collapse = ", "))
    #     cbtable <- cbtable[!toremove, ]
    # }

    ## Add information from custom barcode sets (not implemented)
    customCBsummary <- list()

    ## Create "version info" table
    versiontable <- t(data.frame(
        `Start time` = metainfo$start_time,
        `Salmon version` = metainfo$salmon_version,
        `alevin-fry version (quant)` = quantinfo$version_str,
        `Index` = cmdinfo$index,
        `R1file` = paste(cmdinfo$mates1,
                         collapse = ", "),
        `R2file` = paste(cmdinfo$mates2,
                         collapse = ", "),
        `tgMap` = ifelse(is.null(cmdinfo$tgMap), "NA",
                         cmdinfo$tgMap),
        `Library type` = metainfo$library_types,
        stringsAsFactors = FALSE,
        check.names = FALSE
    ))

    ## Create summary tables
    summarytable_full <- t(data.frame(
        `Total number of processed reads` =
            as.character(metainfo$num_processed),
        `Number of mapped reads` = metainfo$num_mapped,
        `Total number of observed cell barcodes` =
            as.character(length(unique(cbtable$CB))),
        stringsAsFactors = FALSE,
        check.names = FALSE
    ))

    summarytable_permitlist <- .makeSummaryTable(
        cbtable = cbtable,
        colName = "inPermitList",
        cbName = " (permitlist)",
        countCol = "nbrMappedUMI",
        quantmat = NULL
    )


    ## Return
    list(cbTable = cbtable, versionTable = versiontable,
         summaryTables = c(list(fullDataset = summarytable_full,
                                permitlist = summarytable_permitlist),
                           customCBsummary),
         type = "alevin-fry"
    )
}

.readAlevinFryQC_v0.5.0 <- function(mapDir, permitDir, quantDir) {

    ## Raw CB frequencies (in descending order)
    if (file.exists(file.path(permitDir, "all_freq.bin"))) {
        rawcbfreq <- cpp_get_permit_freq_info(file.path(permitDir,
                                                        "all_freq.bin"))
    } else {
        message(file.path(permitDir, "all_freq.bin"),
                " is missing - using permit_freq.bin")
        rawcbfreq <- cpp_get_permit_freq_info(file.path(permitDir,
                                                        "permit_freq.bin"))
    }
    rawcbfreq <- data.frame(CB = rawcbfreq[[1]],
                            originalFreq = rawcbfreq[[2]]) %>%
        dplyr::arrange(dplyr::desc(originalFreq)) %>%
        dplyr::mutate(ranking = seq_len(length(CB)))
    if (!all(diff(rawcbfreq$originalFreq) <= 0)) {
        warning("The raw CB frequencies are not sorted in decreasing order")
    }

    ## FeatureDump
    featuredump <- utils::read.delim(file.path(quantDir, "featureDump.txt"),
                                     header = TRUE, as.is = TRUE, sep = "\t")
    featuredump <- featuredump %>%
        dplyr::rename(mappingRate = MappingRate,
                      collapsedFreq = CorrectedReads,
                      dedupRate = DedupRate,
                      nbrGenesAboveMean = NumGenesOverMean,
                      nbrMappedUMI = MappedReads,
                      totalUMICount = DeduplicatedReads,
                      nbrGenesAboveZero = NumGenesExpressed)

    permitlist <- cpp_get_permit_freq_info(file.path(permitDir,
                                                     "permit_freq.bin"))[[1]]

    ## Meta information and command information
    metainfo <- rjson::fromJSON(file = file.path(mapDir,
                                                 "aux_info/meta_info.json"))
    cmdinfo <- rjson::fromJSON(file = file.path(mapDir, "cmd_info.json"))
    quantinfo <- rjson::fromJSON(file = file.path(quantDir, "quant.json"))
    quantinfo <- .parse_quant_t2gmap(quantinfo)

    ## Merge information about quantified CBs
    cbtable <- dplyr::full_join(
        rawcbfreq,
        featuredump,
        by = "CB"
    )

    cbtable <- cbtable %>%
        dplyr::mutate(inPermitList = CB %in% permitlist)

    ## Check if there is any barcode that is not in the permitlist,
    ## but which has an original ranking lower than any barcode that is
    ## in the permitlist, and remove it.
    # toremove <-
    #     !cbtable$inPermitList &
    #     cbtable$ranking <= max(cbtable$ranking[cbtable$inPermitList])
    # if (any(toremove)) {
    #     warning("Excluding ", sum(toremove), " unquantified barcode",
    #             ifelse(sum(toremove) > 1, "s", ""),
    #             " with higher original frequency than barcodes ",
    #             "included in the permitlist: ",
    #             paste0(cbtable$CB[toremove], collapse = ", "))
    #     cbtable <- cbtable[!toremove, ]
    # }

    ## Add information from custom barcode sets (not implemented)
    customCBsummary <- list()

    ## Create "version info" table
    versiontable <- t(data.frame(
        `Start time` = metainfo$start_time,
        `Salmon version` = metainfo$salmon_version,
        `alevin-fry version (quant)` = quantinfo$version_str,
        `Index` = cmdinfo$index,
        `R1file` = paste(cmdinfo$mates1,
                         collapse = ", "),
        `R2file` = paste(cmdinfo$mates2,
                         collapse = ", "),
        `tgMap` = quantinfo$tgMap,
        `Library type` = metainfo$library_types,
        stringsAsFactors = FALSE,
        check.names = FALSE
    ))

    ## Create summary tables
    summarytable_full <- t(data.frame(
        `Total number of processed reads` =
            as.character(metainfo$num_processed),
        `Number of mapped reads` = metainfo$num_mapped,
        `Total number of observed cell barcodes` =
            as.character(length(unique(cbtable$CB))),
        stringsAsFactors = FALSE,
        check.names = FALSE
    ))

    summarytable_permitlist <- .makeSummaryTable(
        cbtable = cbtable,
        colName = "inPermitList",
        cbName = " (permitlist)",
        countCol = "nbrMappedUMI",
        quantmat = NULL
    )


    ## Return
    list(cbTable = cbtable, versionTable = versiontable,
         summaryTables = c(list(fullDataset = summarytable_full,
                                permitlist = summarytable_permitlist),
                           customCBsummary),
         type = "alevin-fry"
    )
}



.readAlevinFryQC_piscemv0.6.0 <- function(mapDir, permitDir, quantDir) {

    ## Raw CB frequencies (in descending order)
    if (file.exists(file.path(permitDir, "all_freq.bin"))) {
        rawcbfreq <- cpp_get_permit_freq_info(file.path(permitDir,
                                                        "all_freq.bin"))
    } else {
        message(file.path(permitDir, "all_freq.bin"),
                " is missing - using permit_freq.bin")
        rawcbfreq <- cpp_get_permit_freq_info(file.path(permitDir,
                                                        "permit_freq.bin"))
    }
    rawcbfreq <- data.frame(CB = rawcbfreq[[1]],
                            originalFreq = rawcbfreq[[2]]) %>%
        dplyr::arrange(dplyr::desc(originalFreq)) %>%
        dplyr::mutate(ranking = seq_len(length(CB)))
    if (!all(diff(rawcbfreq$originalFreq) <= 0)) {
        warning("The raw CB frequencies are not sorted in decreasing order")
    }

    ## FeatureDump
    featuredump <- utils::read.delim(file.path(quantDir, "featureDump.txt"),
                                     header = TRUE, as.is = TRUE, sep = "\t")
    featuredump <- featuredump %>%
        dplyr::rename(mappingRate = MappingRate,
                      collapsedFreq = CorrectedReads,
                      dedupRate = DedupRate,
                      nbrGenesAboveMean = NumGenesOverMean,
                      nbrMappedUMI = MappedReads,
                      totalUMICount = DeduplicatedReads,
                      nbrGenesAboveZero = NumGenesExpressed)

    permitlist <- cpp_get_permit_freq_info(file.path(permitDir,
                                                     "permit_freq.bin"))[[1]]

    mapinfo <- rjson::fromJSON(file = file.path(mapDir, "map_info.json"))
    mapinfo <- .parse_piscem_map_info(mapinfo)

    quantinfo <- rjson::fromJSON(file = file.path(quantDir, "quant.json"))
    quantinfo <- .parse_quant_t2gmap(quantinfo)


    ## Merge information about quantified CBs
    cbtable <- dplyr::full_join(
        rawcbfreq,
        featuredump,
        by = "CB"
    )

    cbtable <- cbtable %>%
        dplyr::mutate(inPermitList = CB %in% permitlist)

    ## Add information from custom barcode sets (not implemented)
    customCBsummary <- list()


    ## Create "version info" table
    versiontable <- t(data.frame(
        `Run time (seconds)` = mapinfo$runtime_seconds,
        # `Salmon version` = metainfo$salmon_version,
        `alevin-fry version (quant)` = quantinfo$version_str,
        `Index` = mapinfo$Index,
        `R1file` = mapinfo$R1file,
        `R2file` = mapinfo$R2file,
        `tgMap` = quantinfo$tgMap,
        stringsAsFactors = FALSE,
        check.names = FALSE
    ))


    ## Create summary tables
    summarytable_full <- t(data.frame(
        `Total number of processed reads` =
            as.character(mapinfo$num_reads),
        `Number of mapped reads` = mapinfo$num_mapped,
        `Percent of mapped reads` = mapinfo$percent_mapped,
        `Total number of observed cell barcodes` =
            as.character(length(unique(cbtable$CB))),
        stringsAsFactors = FALSE,
        check.names = FALSE
    ))

    summarytable_permitlist <- .makeSummaryTable(
        cbtable = cbtable,
        colName = "inPermitList",
        cbName = " (permitlist)",
        countCol = "nbrMappedUMI",
        quantmat = NULL
    )


    ## Return
    list(cbTable = cbtable, versionTable = versiontable,
         summaryTables = c(list(fullDataset = summarytable_full,
                                permitlist = summarytable_permitlist),
                           customCBsummary),
         type = "alevin-fry"
    )
}

#' @keywords internal
#' @noRd
.parse_piscem_map_info <- function(mapinfo) {
    cmdline <- strsplit(mapinfo$cmdline," ")[[1]][-1]

    mapinfo$Index <- cmdline[which(cmdline == "-i" | cmdline == "--index") + 1]
    mapinfo$R1file <- cmdline[which(cmdline == "-1" | cmdline == "--read1") + 1]
    mapinfo$R2file <- cmdline[which(cmdline == "-2" | cmdline == "--read2") + 1]

    mapinfo
}

#' @keywords internal
#' @noRd
.parse_quant_t2gmap <- function(quantinfo) {
    if (is.null(quantinfo$quant_options$tg_map)) {
        cmd <- strsplit(quantinfo$cmd, " ")[[1]]
        quantinfo$tgMap <- cmd[which(cmd == "-m" | cmd == "--tg-map") + 1]
    } else {
        quantinfo$tgMap <- quantinfo$quant_options$tg_map
    }
    quantinfo
}

#' Knee plot of raw cell barcode frequencies
#'
#' Plot the raw cell barcode frequencies in decreasing order, and indicate a
#' predetermined breakpoint (indicating barcodes included in the original
#' whitelist) using color as well as a label.
#'
#' @author Charlotte Soneson
#'
#' @param cbTable \code{data.frame} with raw barcode frequencies (such as the
#'     \code{cbTable} returned by \code{readAlevinQC} or
#'     \code{readAlevinFryQC}).
#' @param firstSelColName Character scalar indicating the name of the logical
#'     column in \code{cbTable} that corresponds to the original selection of
#'     barcodes for quantification.
#'
#' @export
#'
#' @importFrom ggplot2 ggplot geom_line aes scale_x_log10 xlab ylab theme_bw
#'     theme scale_color_manual geom_label element_text geom_rug
#' @import dplyr
#' @importFrom rlang .data
#'
#' @return A ggplot object
#'
#' @examples
#' alevin <- readAlevinQC(system.file("extdata/alevin_example_v0.14",
#'                                    package = "alevinQC"))
#' plotAlevinKneeRaw(alevin$cbTable)
#'
plotAlevinKneeRaw <- function(cbTable,
                              firstSelColName = "inFirstWhiteList") {
    ## Check input arguments
    .assertVector(x = cbTable, type = "data.frame")
    .assertScalar(x = firstSelColName, type = "character",
                  validValues = colnames(cbTable))
    .assertVector(x = cbTable[[firstSelColName]], type = "logical")
    stopifnot(all(c("ranking", "originalFreq") %in%
                      colnames(cbTable)))

    ## If all the barcodes in the first white list are in the top of the ranking,
    ## make a line plot. Otherwise, indicate the barcodes in the first whitelist
    ## with a rug
    if (max(cbTable$ranking[cbTable[[firstSelColName]]], na.rm = TRUE) <
        min(cbTable$ranking[!cbTable[[firstSelColName]]], na.rm = TRUE)) {
        g <- ggplot2::ggplot(cbTable,
                        ggplot2::aes(x = .data$ranking,
                                     y = .data$originalFreq)) +
            ggplot2::geom_line(linewidth = 2,
                               ggplot2::aes(color = .data[[firstSelColName]])) +
            ggplot2::scale_x_log10() +
            ggplot2::scale_y_log10() +
            ggplot2::xlab("Cell barcode rank") +
            ggplot2::ylab("Cell barcode frequency") +
            ggplot2::theme_bw() +
            ggplot2::theme(legend.position = "none",
                           axis.title = ggplot2::element_text(size = 12)) +
            ggplot2::scale_color_manual(values = c(`TRUE` = "red",
                                                   `FALSE` = "black"))
        if (any(!cbTable[[firstSelColName]])) {
            g <- g + ggplot2::geom_label(
                data = cbTable %>%
                    dplyr::filter(.data[[firstSelColName]]) %>%
                    dplyr::filter(ranking == max(ranking)),
                ggplot2::aes(label = paste0("(", ranking, ", ",
                                            originalFreq, ")")),
                hjust = 0, nudge_x = 0.1)
        }
    } else {
        g <- ggplot2::ggplot(cbTable,
                        ggplot2::aes(x = .data$ranking,
                                     y = .data$originalFreq)) +
            ggplot2::geom_line(linewidth = 2, color = "black") +
            ggplot2::scale_x_log10() +
            ggplot2::scale_y_log10() +
            ggplot2::xlab("Cell barcode rank") +
            ggplot2::ylab("Cell barcode frequency") +
            ggplot2::theme_bw() +
            ggplot2::theme(legend.position = "none",
                           axis.title = ggplot2::element_text(size = 12)) +
            ggplot2::geom_rug(data = cbTable %>%
                                  dplyr::filter(.data[[firstSelColName]]),
                              color = "red")
    }
    g
}

#' Generate alevin/alevin-fry summary shiny app
#'
#' Generate a shiny app summarizing the main aspects of an alevin/alevin-fry
#' quantification run. The app generation assumes that alevin has been run
#' with the --dumpFeatures flag to generate the necessary output files.
#'
#' @param baseDir (Only used for alevin output) Path to the output directory
#'     from the alevin run (should be the directory containing the
#'     \code{alevin} directory).
#' @param mapDir (Only used for alevin-fry output) Path to the output directory
#'     from the salmon alevin run (should be the directory containing the
#'     \code{map.rad} file).
#' @param permitDir (Only used for alevin-fry output) Path to the output
#'     directory from the permit list generation step (should be
#'     the directory containing the \code{all_freq.tsv} file).
#' @param quantDir (Only used for alevin-fry output) Path to the output
#'     directory from the alevin-fry quantification step (should be
#'     the directory containing the \code{alevin} directory).
#' @param simpleafQuantDir (Only used for simpleaf output) Path to the output
#'     directory from the simpleaf run (should be the directory containing the
#'     \code{af_map} and \code{af_quant} directories).
#' @param sampleId Sample ID, will be used set the title for the app.
#' @param customCBList Named list with custom set(s) of barcodes to provide
#'     summary statistics/plots for, in addition to the whitelists generated by
#'     alevin.
#' @param addStopButton Logical scalar. If \code{TRUE} (default), will add a
#'     dropdown menu with a button to stop the app (by calling
#'     \code{shiny::stopApp}) and return a list with the information displayed
#'     in the app.
#'
#' @author Charlotte Soneson
#'
#' @name qcShiny
#'
#' @import dplyr
#' @importFrom shiny fluidRow plotOutput renderPlot shinyApp actionButton icon
#'     stopApp observeEvent
#' @importFrom shinydashboard dashboardPage dashboardHeader dashboardSidebar
#'     box dropdownMenu notificationItem dropdownMenuOutput renderMenu
#' @importFrom DT dataTableOutput datatable renderDataTable
#' @importFrom utils packageVersion
#'
#' @return A shiny app.
#'
#' @examples
#' app <- alevinQCShiny(
#'     baseDir = system.file("extdata/alevin_example_v0.14",
#'                           package = "alevinQC"),
#'     sampleId = "example")
#' if (interactive()) {
#'     shiny::runApp(app)
#' }
#'
#' app <- alevinFryQCShiny(
#'     mapDir = system.file("extdata/alevinfry_example_v0.5.0/map",
#'                          package = "alevinQC"),
#'     permitDir = system.file("extdata/alevinfry_example_v0.5.0/permit",
#'                             package = "alevinQC"),
#'     quantDir = system.file("extdata/alevinfry_example_v0.5.0/quant",
#'                            package = "alevinQC"),
#'     sampleId = "example")
#' if (interactive()) {
#'     shiny::runApp(app)
#' }
#'
#' app <- simpleafQCShiny(
#'     simpleafQuantDir = system.file("extdata/alevinfry_example_piscem_v0.6.0",
#'                                    package = "alevinQC"),
#'     sampleId = "example")
#' if (interactive()) {
#'     shiny::runApp(app)
#' }
#'
NULL

#' @rdname qcShiny
#' @export
alevinQCShiny <- function(baseDir, sampleId, customCBList = list(),
                          addStopButton = TRUE) {
    .alevinQCShiny(baseDir = baseDir, mapDir = NULL, permitDir = NULL,
                   quantDir = NULL, quantMethod = "alevin", sampleId = sampleId,
                   customCBList = customCBList, addStopButton = addStopButton)
}

#' @rdname qcShiny
#' @export
alevinFryQCShiny <- function(mapDir, permitDir, quantDir, sampleId,
                             addStopButton = TRUE) {
    .alevinQCShiny(baseDir = NULL, mapDir = mapDir, permitDir = permitDir,
                   quantDir = quantDir, quantMethod = "alevin-fry",
                   sampleId = sampleId, customCBList = list(),
                   addStopButton = addStopButton)
}

#' @rdname qcShiny
#' @export
simpleafQCShiny <- function(simpleafQuantDir, sampleId, addStopButton = TRUE) {

    mapDir <- file.path(simpleafQuantDir, "af_map")
    permitDir <- file.path(simpleafQuantDir, "af_quant")
    quantDir <- file.path(simpleafQuantDir, "af_quant")

    if (!dir.exists(mapDir)) {
        stop("The provided simpleaf quant output directory doesn't contain ",
             "the `af_map` folder; Cannot proceed. Please run ",
             "alevinFryQCReport() and provide mapDir, permitDir, quantDir ",
             "explicitly. If produced by calling `simpleaf quant`, the ",
             "permitDir and `quantDir` is the same and is named as `af_quant` ",
             "under the simpleaf quant output directory.")
    }

    if (!dir.exists(quantDir)) {
        stop("The provided simpleaf quant output directory doesn't contain ",
             "the `af_quant` folder; Cannot proceed. Please run ",
             "alevinFryQCReport() and provide mapDir, permitDir, quantDir ",
             "explicitly. If produced by calling `simpleaf quant`, the ",
             "permitDir and `quantDir` is the same and is named as `af_quant` ",
             "under the simpleaf quant output directory.")
    }

    .alevinQCShiny(baseDir = NULL, mapDir = mapDir, permitDir = permitDir,
                   quantDir = quantDir, quantMethod = "alevin-fry",
                   sampleId = sampleId, customCBList = list(),
                   addStopButton = addStopButton)
}


#' @keywords internal
#' @noRd
.alevinQCShiny <- function(baseDir, mapDir, permitDir, quantDir,
                           quantMethod, sampleId, customCBList = list(),
                           addStopButton) {
    if (quantMethod == "alevin") {
        checkAlevinInputFiles(baseDir)
        alevin <- readAlevinQC(baseDir = baseDir, customCBList = customCBList)
        firstSelColName <- "inFirstWhiteList"
        wlName <- "initial whitelist"
        countCol <- "collapsedFreq"
    } else if (quantMethod == "alevin-fry") {
        checkAlevinFryInputFiles(mapDir = mapDir, permitDir = permitDir,
                                 quantDir = quantDir)
        alevin <- readAlevinFryQC(mapDir = mapDir, permitDir = permitDir,
                                  quantDir = quantDir)
        firstSelColName <- "inPermitList"
        wlName <- "permitlist"
        countCol <- "nbrMappedUMI"
    }



    pLayout <- shinydashboard::dashboardPage(
        skin = "red",

        shinydashboard::dashboardHeader(
            title = paste0("alevinQC (v",
                           utils::packageVersion("alevinQC"), "), ",
                           sampleId),
            titleWidth = (10 + nchar(sampleId)) * 20,
            shinydashboard::dropdownMenuOutput("stopAppMenu")
        ),

        shinydashboard::dashboardSidebar(disable = TRUE),

        shinydashboard::dashboardBody(
            shiny::fluidRow(
                shinydashboard::box(
                    width = 6,
                    title = paste0("Version info, ", quantMethod, " run"),
                    DT::dataTableOutput("versionTable")
                ),
                shinydashboard::box(
                    width = 6,
                    title = "Summary tables",
                    if (quantMethod == "alevin") DT::dataTableOutput("summaryTableFull"),
                    if (quantMethod == "alevin") DT::dataTableOutput("summaryTableInitialWl"),
                    if (quantMethod == "alevin") DT::dataTableOutput("summaryTableFinalWl"),
                    if (quantMethod == "alevin-fry") DT::dataTableOutput("summaryTablePl"),
                    shiny::uiOutput("summaryTablesCustomCBs")
                )

            ),
            shiny::fluidRow(
                shinydashboard::box(
                    width = 4,
                    title = "Knee plot, initial whitelist determination",
                    shiny::plotOutput("rawCBKneePlot")
                ),
                shinydashboard::box(
                    width = 4,
                    title = "Barcode collapsing",
                    shiny::plotOutput("barcodeCollapsePlot")
                ),
                shinydashboard::box(
                    width = 4,
                    title = "Knee plot, number of genes per cell",
                    shiny::plotOutput("nbrGenesKneePlot")
                )
            ),
            shiny::fluidRow(
                shinydashboard::box(
                    width = 12,
                    title = paste0("Quantification summary (", wlName, ")"),
                    shiny::plotOutput("quantPlot"),
                    shiny::uiOutput("quantPlotsCustomCBs")
                )
            ),
            shiny::fluidRow(
                shinydashboard::box(
                    width = 12,
                    title = paste0("Selected summary distributions (",
                                   wlName, ")"),
                    shiny::plotOutput("histPlot"),
                    shiny::uiOutput("histPlotsCustomCBs")
                )
            )

        )
    )

    server_function <- function(input, output, session) { # nocov start
        ## ----------------------------------------------------------------- ##
        ## Version table
        ## ----------------------------------------------------------------- ##
        output$versionTable <- DT::renderDataTable(
            DT::datatable(
                alevin$versionTable,
                colnames = "",
                options = list(scrollX = TRUE)
            )
        )

        ## ----------------------------------------------------------------- ##
        ## Standard summary tables
        ## ----------------------------------------------------------------- ##
        output$summaryTableFull <- DT::renderDataTable(
            DT::datatable(
                alevin$summaryTables$fullDataset,
                colnames = "",
                options = list(scrollX = TRUE)
            )
        )

        output$summaryTableInitialWl <- DT::renderDataTable(
            DT::datatable(
                alevin$summaryTables$initialWhitelist,
                colnames = "",
                options = list(scrollX = TRUE)
            )
        )

        output$summaryTableFinalWl <- DT::renderDataTable(
            DT::datatable(
                alevin$summaryTables$finalWhitelist,
                colnames = "",
                options = list(scrollX = TRUE)
            )
        )

        output$summaryTablePl <- DT::renderDataTable(
            DT::datatable(
                alevin$summaryTables$permitlist,
                colnames = "",
                options = list(scrollX = TRUE)
            )
        )

        ## ----------------------------------------------------------------- ##
        ## Custom CB summary tables
        ## ----------------------------------------------------------------- ##
        lapply(seq_along(customCBList), function(i) {
            id <- paste0("dt", i)
            output[[id]] <- DT::renderDataTable(
                DT::datatable(
                    alevin$summaryTables[[paste0("customCB__",
                                                 names(customCBList)[i])]],
                    colnames = "",
                    options = list(scrollX = TRUE)
                )
            )
        })

        output$summaryTablesCustomCBs <- shiny::renderUI({
            lapply(as.list(seq_along(customCBList)), function(i) {
                id <- paste0("dt", i)
                DT::dataTableOutput(id)
            })
        })

        ## ----------------------------------------------------------------- ##
        ## Standard plots
        ## ----------------------------------------------------------------- ##
        output$rawCBKneePlot <- shiny::renderPlot(
            plotAlevinKneeRaw(alevin$cbTable, firstSelColName = firstSelColName)
        )

        output$barcodeCollapsePlot <- shiny::renderPlot(
            plotAlevinBarcodeCollapse(alevin$cbTable,
                                      firstSelColName = firstSelColName,
                                      countCol = countCol)
        )

        output$nbrGenesKneePlot <- shiny::renderPlot(
            plotAlevinKneeNbrGenes(alevin$cbTable,
                                   firstSelColName = firstSelColName)
        )

        ## ----------------------------------------------------------------- ##
        ## Standard quant plots
        ## ----------------------------------------------------------------- ##
        output$quantPlot <- shiny::renderPlot(
            if (quantMethod == "alevin") {
                plotAlevinQuant(alevin$cbTable, colName = "inFinalWhiteList",
                                cbName = "final whitelist",
                                firstSelColName = firstSelColName)
            } else if (quantMethod == "alevin-fry") {
                plotAlevinQuant(alevin$cbTable, colName = "inPermitList",
                                cbName = "permitlist",
                                firstSelColName = firstSelColName)

            }
        )

        ## ----------------------------------------------------------------- ##
        ## Custom CB quant plots
        ## ----------------------------------------------------------------- ##
        lapply(seq_along(customCBList), function(i) {
            id <- paste0("qpl", i)
            output[[id]] <- shiny::renderPlot(
                plotAlevinQuant(alevin$cbTable,
                                colName = paste0("customCB__",
                                                 names(customCBList)[i]),
                                cbName = names(customCBList)[i],
                                firstSelColName = firstSelColName)
            )
        })

        output$quantPlotsCustomCBs <- shiny::renderUI({
            lapply(as.list(seq_along(customCBList)), function(i) {
                id <- paste0("qpl", i)
                shiny::plotOutput(id)
            })
        })

        ## ----------------------------------------------------------------- ##
        ## Standard distribution plots
        ## ----------------------------------------------------------------- ##
        output$histPlot <- shiny::renderPlot(
            if (quantMethod == "alevin") {
                cowplot::plot_grid(
                    plotAlevinHistogram(alevin$cbTable, plotVar = "dedupRate",
                                        axisLabel = "Deduplication rate",
                                        colName = "inFinalWhiteList",
                                        cbName = "final whitelist",
                                        firstSelColName = firstSelColName),
                    plotAlevinHistogram(alevin$cbTable, plotVar = "mappingRate",
                                        axisLabel = "Mapping rate",
                                        colName = "inFinalWhiteList",
                                        cbName = "final whitelist",
                                        firstSelColName = firstSelColName)
                )
            } else if (quantMethod == "alevin-fry") {
                cowplot::plot_grid(
                    plotAlevinHistogram(alevin$cbTable, plotVar = "dedupRate",
                                        axisLabel = "Deduplication rate",
                                        colName = firstSelColName,
                                        cbName = "permitlist",
                                        firstSelColName = firstSelColName),
                    plotAlevinHistogram(alevin$cbTable, plotVar = "mappingRate",
                                        axisLabel = "Mapping rate",
                                        colName = firstSelColName,
                                        cbName = "permitlist",
                                        firstSelColName = firstSelColName)
                )

            }
        )

        ## ----------------------------------------------------------------- ##
        ## Custom CB distribution plots
        ## ----------------------------------------------------------------- ##
        lapply(seq_along(customCBList), function(i) {
            id <- paste0("hpl", i)
            output[[id]] <- shiny::renderPlot(
                cowplot::plot_grid(
                    plotAlevinHistogram(
                        alevin$cbTable, plotVar = "dedupRate",
                        axisLabel = "Deduplication rate",
                        colName = paste0("customCB__",
                                         names(customCBList)[i]),
                        cbName = names(customCBList)[i]),
                    plotAlevinHistogram(
                        alevin$cbTable, plotVar = "mappingRate",
                        axisLabel = "Mapping rate",
                        colName = paste0("customCB__",
                                         names(customCBList)[i]),
                        cbName = names(customCBList)[i])
                )
            )
        })

        output$histPlotsCustomCBs <- shiny::renderUI({
            lapply(as.list(seq_along(customCBList)), function(i) {
                id <- paste0("hpl", i)
                shiny::plotOutput(id)
            })
        })

        ## ----------------------------------------------------------------- ##
        ## Close and return button
        ## ----------------------------------------------------------------- ##
        output$stopAppMenu <- shinydashboard::renderMenu(
            if (addStopButton) {
                shinydashboard::dropdownMenu(
                    type = "tasks",
                    icon = shiny::icon("circle-xmark"),
                    badgeStatus = NULL,
                    headerText = "",
                    shinydashboard::notificationItem(
                        text = shiny::actionButton(
                            inputId = "close_app", label = "Close app",
                            icon = shiny::icon("circle-xmark")
                        ),
                        icon = shiny::icon(""), # tricking it to not have additional icon
                        status = "primary"
                    )
                )
            } else {
                NULL
            }
        )
        shiny::observeEvent(input$close_app, {
            shiny::stopApp(returnValue = list(alevin = alevin,
                                              firstSelColName = firstSelColName,
                                              wlName = wlName,
                                              countCol = countCol))
        })

    } # nocov end

    shiny::shinyApp(ui = pLayout, server = server_function)
}

#' Check whether pandoc and pandoc-citeproc are available
#'
#' @author Charlotte Soneson
#'
#' @param ignorePandoc logical. If TRUE, just give a warning if pandoc or
#'     pandoc-citeproc is not available. If FALSE, stop.
#'
#' @keywords internal
#' @noRd
#'
#' @return A logical(1), indicating whether pandoc can be run or not.
#'     In addition, raises either a warning or an error (depending on the
#'     value of \code{ignorePandoc}) if pandoc or pandoc-citeproc is not
#'     available.
#'
#' @importFrom rmarkdown pandoc_available pandoc_exec
#'
.checkPandoc <- function(ignorePandoc) {
    # nocov start
    ## Initialize output to TRUE
    doRender <- TRUE

    ## First check whether pandoc is available
    if (!rmarkdown::pandoc_available()) {
        doRender <- FALSE
        ## If pandoc is not available, either give a warning or an error,
        ## depending on the value of ignorePandoc
        if (ignorePandoc) {
            ## If ignorePandoc is TRUE, just give a warning
            warning("pandoc is not available! ",
                    "The final report will not be generated.",
                    immediate. = TRUE)
        } else {
            ## If ignorePandoc is FALSE, stop
            stop("pandoc is not available!")
        }
    } else {
        ## If pandoc is available, check for pandoc-citeproc
        ## Only do this if the pandoc version is <2.11, since
        ## pandoc-citeproc is not included (or needed) in v2.11 and later.
        if (!rmarkdown::pandoc_available(version = "2.11")) {
            ## TRUE if the available pandoc version is not 2.11 or newer
            ## pandoc-citeproc should be found in the path, or in the
            ## same folder as the pandoc executable
            if (Sys.which("pandoc-citeproc") == "" &&
                !file.exists(file.path(dirname(rmarkdown::pandoc_exec()),
                                       "pandoc-citeproc"))) {
                doRender <- FALSE
                ## pandoc-citeproc is required, but not found
                if (ignorePandoc) {
                    ## If ignorePandoc is TRUE, just give a warning
                    warning("pandoc-citeproc is not available! ",
                            "The final report will not be generated.",
                            immediate. = TRUE)
                } else {
                    ## If ignorePandoc is FALSE, stop
                    stop("pandoc-citeproc is not available!")
                }
            }
        }
    }
    return(doRender)
    # nocov end
}

#' Generate alevin/alevin-fry summary report
#'
#' Generate a report summarizing the main aspects of an alevin/alevin-fry
#' quantification run. The report generation assumes that alevin/alevin-fry
#' has been run with the --dumpFeatures flag to generate the necessary
#' output files.
#'
#' @param baseDir (Only used for alevin output) Path to the output directory
#'     from the alevin run (should be the directory containing the
#'     \code{alevin} directory).
#' @param mapDir (Only used for alevin-fry output) Path to the output directory
#'     from the salmon alevin run (should be the directory containing the
#'     \code{map.rad} file).
#' @param permitDir (Only used for alevin-fry output) Path to the output
#'     directory from the permit list generation step (should be
#'     the directory containing the \code{all_freq.tsv} file).
#' @param quantDir (Only used for alevin-fry output) Path to the output
#'     directory from the alevin-fry quantification step (should be
#'     the directory containing the \code{alevin} directory).
#' @param simpleafQuantDir (Only used for simpleaf output) Path to the output
#'     directory from the simpleaf run (should be the directory containing the
#'     \code{af_map} and \code{af_quant} directories).
#' @param sampleId Sample ID, will be used to set the title for the report.
#' @param outputFile File name of the output report. The file name extension
#'     must be either \code{.html} or \code{.pdf}, and consistent with the value
#'     of \code{outputFormat}.
#' @param outputDir Path to the output directory where the report will be
#'     generated.
#' @param outputFormat The format of the output report. Either
#'     \code{"html_document"}, \code{"pdf_document"},
#'     \code{"BiocStyle::html_document"} or \code{"BiocStyle::pdf_document"}.
#'     The file name extension of \code{outputFile} must be consistent with this
#'     choice.
#' @param showCode Logical, whether to display the R code in the report.
#' @param forceOverwrite Logical, whether to force overwrite an existing report
#'     with the same name in the output directory.
#' @param knitrProgress Logical, whether to display the progress of \code{knitr}
#'     when generating the report.
#' @param quiet Logical, whether to show progress messages.
#' @param ignorePandoc Logical, determines what to do if \code{pandoc} or
#'     \code{pandoc-citeproc} is missing (if \code{Sys.which("pandoc")} or
#'     \code{Sys.which("pandoc-citeproc")} returns ""). If \code{ignorePandoc}
#'     is TRUE, only a warning is given. The figures will be generated, but not
#'     the final report. If \code{ignorePandoc} is FALSE (default), the
#'     execution stops immediately.
#' @param customCBList Named list with custom set(s) of barcodes to provide
#'     summary statistics/plots for, in addition to the whitelists generated by
#'     alevin.
#' @param ... Other arguments that will be passed to \code{rmarkdown::render}.
#'
#' @author Charlotte Soneson
#'
#' @details When the function is called, a .Rmd template file will be copied
#'     into the output directory, and \code{rmarkdown::render} will be called to
#'     generate the final report. If there is already a .Rmd file with the same
#'     name in the output directory, the function will raise an error and stop,
#'     to avoid overwriting the existing file. The reason for this behaviour is
#'     that the copied template in the output directory will be deleted once
#'     the report is generated.
#'
#' @name qcReport
#'
#' @importFrom rmarkdown render
#' @importFrom tools file_ext file_path_sans_ext
#' @importFrom methods is
#' @import dplyr
#'
#' @return Generates a summary report in the \code{outputDir} directory, and
#'     returns (invisibly) the name of the generated report.
#'
#' @examples
#' alevinQCReport(
#'     baseDir = system.file("extdata/alevin_example_v0.14",
#'                           package = "alevinQC"),
#'     sampleId = "example", outputFile = "alevinReport.html",
#'     outputDir = tempdir(), forceOverwrite = TRUE)
#'
#' alevinFryQCReport(
#'     mapDir = system.file("extdata/alevinfry_example_v0.5.0/map",
#'                          package = "alevinQC"),
#'     permitDir = system.file("extdata/alevinfry_example_v0.5.0/permit",
#'                             package = "alevinQC"),
#'     quantDir = system.file("extdata/alevinfry_example_v0.5.0/quant",
#'                            package = "alevinQC"),
#'     sampleId = "example", outputFile = "alevinFryReport.html",
#'     outputDir = tempdir(), forceOverwrite = TRUE)
#'
#' simpleafQCReport(
#'     simpleafQuantDir = system.file("extdata/alevinfry_example_piscem_v0.6.0",
#'                                    package = "alevinQC"),
#'     sampleId = "example", outputFile = "simpleafReport.html",
#'     outputDir = tempdir(), forceOverwrite = TRUE)
#'
NULL

#' @rdname qcReport
#' @export
alevinQCReport <- function(baseDir, sampleId, outputFile, outputDir = "./",
                           outputFormat = NULL, showCode = FALSE,
                           forceOverwrite = FALSE, knitrProgress = FALSE,
                           quiet = FALSE, ignorePandoc = FALSE,
                           customCBList = list(), ...) {
    .alevinQCReport(baseDir = baseDir, mapDir = NULL, permitDir = NULL,
                    quantDir = NULL, quantMethod = "alevin",
                    sampleId = sampleId, outputFile = outputFile,
                    outputDir = outputDir, outputFormat = outputFormat,
                    showCode = showCode, forceOverwrite = forceOverwrite,
                    knitrProgress = knitrProgress, quiet = quiet,
                    ignorePandoc = ignorePandoc,
                    customCBList = customCBList, ...)
}


#' @rdname qcReport
#' @export
simpleafQCReport <- function(simpleafQuantDir, sampleId,
                             outputFile, outputDir = "./",
                             outputFormat = NULL, showCode = FALSE,
                             forceOverwrite = FALSE, knitrProgress = FALSE,
                             quiet = FALSE, ignorePandoc = FALSE,
                             customCBList = list(), ...) {
    if (length(customCBList) != 0) {
        warning("custom CB lists are currently not implemented for ",
                "alevin-fry QC reports")
    }
    mapDir <- file.path(simpleafQuantDir, "af_map")
    permitDir <- file.path(simpleafQuantDir, "af_quant")
    quantDir <- file.path(simpleafQuantDir, "af_quant")

    if (!dir.exists(mapDir)) {
        stop("The provided simpleaf quant output directory doesn't contain ",
             "the `af_map` folder; Cannot proceed. Please run ",
             "alevinFryQCReport() and provide mapDir, permitDir, quantDir ",
             "explicitly. If produced by calling `simpleaf quant`, the ",
             "permitDir and `quantDir` is the same and is named as `af_quant` ",
             "under the simpleaf quant output directory.")
    }

    if (!dir.exists(quantDir)) {
        stop("The provided simpleaf quant output directory doesn't contain ",
             "the `af_quant` folder; Cannot proceed. Please run ",
             "alevinFryQCReport() and provide mapDir, permitDir, quantDir ",
             "explicitly. If produced by calling `simpleaf quant`, the ",
             "permitDir and `quantDir` is the same and is named as `af_quant` ",
             "under the simpleaf quant output directory.")
    }

    .alevinQCReport(baseDir = NULL, mapDir = mapDir, permitDir = permitDir,
                    quantDir = quantDir, quantMethod = "alevin-fry",
                    sampleId = sampleId, outputFile = outputFile,
                    outputDir = outputDir, outputFormat = outputFormat,
                    showCode = showCode, forceOverwrite = forceOverwrite,
                    knitrProgress = knitrProgress, quiet = quiet,
                    ignorePandoc = ignorePandoc,
                    customCBList = list(), ...)
}


#' @rdname qcReport
#' @export
alevinFryQCReport <- function(mapDir, permitDir, quantDir, sampleId,
                              outputFile, outputDir = "./",
                              outputFormat = NULL, showCode = FALSE,
                              forceOverwrite = FALSE, knitrProgress = FALSE,
                              quiet = FALSE, ignorePandoc = FALSE,
                              customCBList = list(), ...) {
    if (length(customCBList) != 0) {
        warning("custom CB lists are currently not implemented for ",
                "alevin-fry QC reports")
    }
    .alevinQCReport(baseDir = NULL, mapDir = mapDir, permitDir = permitDir,
                    quantDir = quantDir, quantMethod = "alevin-fry",
                    sampleId = sampleId, outputFile = outputFile,
                    outputDir = outputDir, outputFormat = outputFormat,
                    showCode = showCode, forceOverwrite = forceOverwrite,
                    knitrProgress = knitrProgress, quiet = quiet,
                    ignorePandoc = ignorePandoc,
                    customCBList = list(), ...)
}

#' @keywords internal
#' @noRd
#' @author Charlotte Soneson
.alevinQCReport <- function(baseDir, mapDir, permitDir, quantDir,
                            quantMethod, sampleId, outputFile, outputDir = "./",
                            outputFormat = NULL, showCode = FALSE,
                            forceOverwrite = FALSE, knitrProgress = FALSE,
                            quiet = FALSE, ignorePandoc = FALSE,
                            customCBList = list(), ...) {
    ## This function was inspired by code from Nicholas Hamilton, provided at
    ## http://stackoverflow.com/questions/37097535/generate-report-in-r

    ## If possible, set output format based on the extension of outputFile, if
    ## the output format is not provided
    if (is.null(outputFormat)) {
        if (tools::file_ext(outputFile) == "pdf") {
            outputFormat <- "pdf_document"
        } else {
            outputFormat <- "html_document"
        }
    }

    ## Check if pandoc and pandoc-citeproc are available
    doRender <- .checkPandoc(ignorePandoc)

    ## ---------------------------------------------------------------------- ##
    ## --------------------- Check input arguments -------------------------- ##
    ## ---------------------------------------------------------------------- ##

    ## ------------------------ outputFormat -------------------------------- ##
    ## Raise an error if outputFormat is not one of the allowed
    if (!(outputFormat %in% c("pdf_document", "html_document",
                              "BiocStyle::html_document",
                              "BiocStyle::pdf_document"))) {
        stop("The provided outputFormat is currently not supported. Please ",
             "use either 'html_document', 'pdf_document', ",
             "'BiocStyle::html_document' or 'BiocStyle::pdf_document'.",
             call. = FALSE)
    }

    ## Raise an error if the output format and file name extension don't match
    if (gsub("BiocStyle::", "", outputFormat) !=
        paste0(tools::file_ext(outputFile), "_document")) {
        stop("File name extension of outputFile (.",
             tools::file_ext(outputFile),
             ") doesn't agree with the ",
             "outputFormat, should be .",
             sub("BiocStyle::", "", sub("_document$", "",
                                        outputFormat)), call. = FALSE)
    }

    ## ----------------------- input directory ------------------------------ ##
    ## Normalize directory paths
    if (!is.null(baseDir)) {
        baseDir <- normalizePath(baseDir)
    }
    if (!is.null(mapDir)) {
        mapDir <- normalizePath(mapDir)
    }
    if (!is.null(permitDir)) {
        permitDir <- normalizePath(permitDir)
    }
    if (!is.null(quantDir)) {
        quantDir <- normalizePath(quantDir)
    }


    ## Check that all required input files are available
    if (quantMethod == "alevin") {
        checkAlevinInputFiles(baseDir)
    } else if (quantMethod == "alevin-fry") {
        checkAlevinFryInputFiles(mapDir = mapDir, permitDir = permitDir,
                                 quantDir = quantDir)
    }

    ## sampleId must be a character string of length 1
    if (!is(sampleId, "character") || length(sampleId) != 1) {
        stop("sampleId must be a character string")
    }

    ## --------------------- custom barcode list ---------------------------- ##
    if (length(customCBList) > 0) {
        if (!is(customCBList, "list")) {
            stop("'customCBList' must be a list")
        }
        if (any(is.null(names(customCBList))) ||
            any(names(customCBList) == "")) {
            stop("'customCBList' must be a named list")
        }
        if (!all(vapply(customCBList, function(cbl) is(cbl, "character"),
                        FALSE))) {
            stop("'customCBList' must be a named list of character vectors")
        }
    }

    ## ------------------------- output files ------------------------------- ##
    outputReport <- file.path(outputDir, basename(outputFile))
    outputRmd <- file.path(
        outputDir,
        paste0(tools::file_path_sans_ext(basename(outputFile)), ".Rmd")
    )

    ## Report
    if (file.exists(outputReport)) {
        if (!forceOverwrite) {
            stop("The file ", outputReport,
                 " already exists. Please remove or rename the file, provide ",
                 "another value of outputFile, or set forceOverwrite = TRUE.",
                 call. = FALSE)
        } else {
            if (!quiet) {
                warning("The file ", outputReport,
                        " already exists and will be overwritten, since ",
                        "forceOverwrite = TRUE.", immediate. = TRUE,
                        call. = FALSE)
            }
        }
    }

    ## ------------------------- Rmd template ------------------------------- ##
    ## Path to the template file
    templateFile <- system.file("extdata",
                                "alevin_report_template.Rmd",
                                package = "alevinQC")
    if (file.exists(templateFile)) {
        if (file.exists(outputRmd)) {
            stop("There is already an .Rmd file ", outputRmd,
                 ". Please remove or rename this file, or choose another ",
                 "outputFile name.", call. = FALSE)
        } else {
            file.copy(from = templateFile, to = outputRmd, overwrite = FALSE)
        }
    } else {
        stop("The Rmd template file ", templateFile, " does not exist.",
             call. = FALSE)
    }

    ## ------------------- Fix output file name if needed ------------------- ##
    ## For BiocStyle::pdf_document, we need to supply the output file name
    ## without the extension
    ## (see https://github.com/Bioconductor/BiocStyle/issues/73)
    if (outputFormat == "BiocStyle::pdf_document") {
        outputFile <- tools::file_path_sans_ext(outputFile)
    }

    ## ---------------------------------------------------------------------- ##
    ## ----------------------- Process the arguments ------------------------ ##
    ## ---------------------------------------------------------------------- ##

    args <- list(...)
    args$input <- outputRmd
    args$output_format <- outputFormat
    args$output_file <- outputFile
    args$quiet <- !knitrProgress
    args$run_pandoc <- doRender

    ## ---------------------------------------------------------------------- ##
    ## ------------------------ Render the report --------------------------- ##
    ## ---------------------------------------------------------------------- ##

    outputFile <- do.call("render", args = args)

    ## ---------------------------------------------------------------------- ##
    ## --------------------- Remove temporary file -------------------------- ##
    ## ---------------------------------------------------------------------- ##

    file.remove(outputRmd)

    invisible(outputFile)
}

#' Check that all required input files are available
#'
#' @param baseDir Path to the output directory from the alevin run (should be
#'   the directory containing the \code{alevin} directory).
#'
#' @author Charlotte Soneson
#'
#' @export
#'
#' @importFrom utils read.delim
#'
#' @return Returns nothing, raises an error if any of the required files are
#'   missing.
#'
#' @examples
#' checkAlevinInputFiles(system.file("extdata/alevin_example_v0.14",
#'                                   package = "alevinQC"))
#'
checkAlevinInputFiles <- function(baseDir) {
    msg <- NULL

    ## First check whether the baseDir files are compatible with alevin
    ## v0.14 or later, including a whitelist.txt file.
    ## This is the "standard" v0.14 output.
    v0.14 <- .checkAlevinInputFiles_v0.14(baseDir, type = "standard")
    if (is.null(v0.14)) {
        return("v0.14")
    } else {
        msg <- c(msg,
                 "Input directory not compatible with Salmon v0.14 or newer ",
                 "(without external whitelist), ",
                 "the following required file(s) are missing or malformed:\n",
                 paste(v0.14, collapse = "\n"), "\n\n")
    }

    ## If the above check failed, check whether the baseDir files are
    ## compatible with alevin v0.14 or later, with an external whitelist
    ## provided. In this case, there would not have to be a whitelist.txt
    ## file, and there would be a "whitelist" entry in the meta_info.json file.
    ## This is the "extwl" v0.14 output.
    v0.14extwl <- .checkAlevinInputFiles_v0.14(baseDir, type = "extwl")
    if (is.null(v0.14extwl)) {
        return("v0.14extwl")
    } else {
        msg <- c(msg,
                 "Input directory not compatible with Salmon v0.14 or newer ",
                 "(with external whitelist), ",
                 "the following required file(s) are missing or malformed:\n",
                 paste(v0.14extwl, collapse = "\n"), "\n\n")
    }

    ## If the above checks failed, check whether the baseDir files are
    ## compatible with alevin v0.14 or later, but without the whitelist.txt
    ## file. This can happen if the whitelisting could not be performed
    ## for some reason. This is the "nowl" v0.14 output.
    v0.14nowl <- .checkAlevinInputFiles_v0.14(baseDir, type = "nowl")
    if (is.null(v0.14nowl)) {
        return("v0.14nowl")
    } else {
        msg <- c(msg,
                 "Input directory not compatible with Salmon v0.14 or newer ",
                 "(without final whitelist), ",
                 "the following required file(s) are missing or malformed:\n",
                 paste(v0.14nowl, collapse = "\n"), "\n\n")
    }

    ## If all the checks above failed, check whether the baseDir files are
    ## compatible with alevin pre-v0.14.
    pre0.14 <- .checkAlevinInputFiles_pre0.14(baseDir)
    if (is.null(pre0.14)) {
        return("pre0.14")
    } else {
        msg <- c(msg,
                 "Input directory not compatible with Salmon pre-v0.14, ",
                 "the following required file(s) are missing or malformed:\n",
                 paste(pre0.14, collapse = "\n"), "\n\n")
    }

    ## If we are here, the baseDir doesn't match any of the known
    ## output structures.
    stop(msg)
}


.checkAlevinInputFiles_pre0.14 <- function(baseDir) {
    ## Raise an error if any of the required files is missing
    reqFiles <- c(file.path(baseDir, "alevin/raw_cb_frequency.txt"),
                  file.path(baseDir, "alevin/filtered_cb_frequency.txt"),
                  file.path(baseDir, "alevin/featureDump.txt"),
                  file.path(baseDir, "alevin/MappedUmi.txt"),
                  file.path(baseDir, "alevin/whitelist.txt"),
                  file.path(baseDir, "alevin/quants_mat_rows.txt"),
                  file.path(baseDir, "alevin/quants_mat_cols.txt"),
                  file.path(baseDir, "alevin/quants_mat.gz"),
                  file.path(baseDir, "aux_info/meta_info.json"),
                  file.path(baseDir, "cmd_info.json"))
    missingFiles <- reqFiles[vapply(reqFiles,
                                    function(f) {
                                        !file.exists(f)
                                    }, TRUE)]
    if (length(missingFiles) > 0) {
        return(missingFiles)
    } else {
        return(NULL)
    }
}

# type is either 'standard', 'extwl', or 'nowl'
.checkAlevinInputFiles_v0.14 <- function(baseDir, type = "standard") {
    ## Raise an error if any of the required files is missing
    reqFiles <- c(file.path(baseDir, "alevin/raw_cb_frequency.txt"),
                  file.path(baseDir, "alevin/featureDump.txt"),
                  file.path(baseDir, "aux_info/meta_info.json"),
                  file.path(baseDir, "aux_info/alevin_meta_info.json"),
                  file.path(baseDir, "cmd_info.json"))
    if (type == "standard") {
        ## 'standard' checks should include also the whitelist.txt file
        reqFiles <- c(reqFiles, file.path(baseDir, "alevin/whitelist.txt"))
    }
    ## If any of the files are missing, stop and return these
    missingFiles <- reqFiles[vapply(reqFiles,
                                    function(f) {
                                        !file.exists(f)
                                    }, TRUE)]
    if (length(missingFiles) > 0) {
        return(missingFiles)
    }

    ## All required files exist. Check that they contain the required info.
    if (type == "extwl") {
        ## 'extwl' checks should have a 'whitelist' entry in meta_info.json
        meta <- rjson::fromJSON(file = file.path(baseDir,
                                                 "cmd_info.json"))
        if (is.null(meta$whitelist)) {
            missingFiles <- paste0(
                file.path(baseDir, "aux_info/meta_info.json"),
                " (whitelist entry)"
            )
        }
        if (length(missingFiles) > 0) {
            return(missingFiles)
        }
    }

    ## Check that all required columns in featureDump.txt are present
    coln <- unlist(utils::read.delim(
        file.path(baseDir, "alevin/featureDump.txt"),
        header = FALSE, as.is = TRUE, sep = "\t", nrows = 1))
    if (!(all(c("CorrectedReads", "MappingRate", "DedupRate",
                "NumGenesOverMean", "MappedReads", "DeduplicatedReads",
                "NumGenesExpressed") %in% coln))) {
        missingFiles <- c(file.path(baseDir, "alevin/featureDump.txt"))
    }

    if (length(missingFiles) > 0) {
        return(missingFiles)
    }

    ## If we are here, the input is consistent with the provided
    ## 'type' argument.
    return(NULL)
}

#' Check that all required input files are available for alevin-fry
#'
#' @param mapDir Path to the directory containing the map.rad file
#' @param permitDir Path to the output directory of the alevin-fry
#'     generate-permit-list command.
#' @param quantDir Path to the output of the alevin-fry quant command.
#'
#' @author Charlotte Soneson
#'
#' @export
#'
#' @importFrom utils read.delim
#'
#' @return Returns nothing, raises an error if any of the required files are
#'   missing.
#'
#' @examples
#' checkAlevinFryInputFiles(
#'     mapDir = system.file("extdata/alevinfry_example_v0.5.0/map",
#'                          package = "alevinQC"),
#'     permitDir = system.file("extdata/alevinfry_example_v0.5.0/permit",
#'                             package = "alevinQC"),
#'     quantDir = system.file("extdata/alevinfry_example_v0.5.0/quant",
#'                            package = "alevinQC"))
#'
checkAlevinFryInputFiles <- function(mapDir, permitDir, quantDir) {
    msg <- NULL

    ## First check if the files are compatible with piscem v0.6.0 or newer
    piscem_0.6.0 <- .checkAlevinFryInputFiles_piscem0.6.0(mapDir = mapDir,
                                                          permitDir = permitDir,
                                                          quantDir = quantDir)
    if (is.null(piscem_0.6.0)) {
        return("piscem_v0.6.0")
    } else {
        msg <- c(msg,
                 "Input directory not compatible with alevin-fry with the ",
                 "piscem mapper v0.6.0 or newer, the following required ",
                 "file(s) are missing or malformed:\n",
                 paste(piscem_0.6.0, collapse = "\n"), "\n\n")
    }

    ## If not compatible with piscem v0.6.0, check if the files are
    ## compatible with alevin-fry v0.5.0 or newer
    v0.5.0 <- .checkAlevinFryInputFiles_0.5.0(mapDir = mapDir,
                                              permitDir = permitDir,
                                              quantDir = quantDir)

    if (is.null(v0.5.0)) {
        return("v0.5.0")
    } else {
        msg <- c(msg,
                 "Input directory not compatible with alevin-fry v0.5.0 ",
                 "or newer, the following required file(s) are missing ",
                 "or malformed:\n",
                 paste(v0.5.0, collapse = "\n"), "\n\n")

    }

    ## If not compatible with v0.5.0, check for compatibility with v0.4.3
    v0.4.3 <- .checkAlevinFryInputFiles_0.4.3(mapDir = mapDir,
                                              permitDir = permitDir,
                                              quantDir = quantDir)
    if (is.null(v0.4.3)) {
        return("v0.4.3")
    } else {
        msg <- c(msg,
                 "Input directory not compatible with alevin-fry v0.4.3 ",
                 "or newer, the following required file(s) are missing ",
                 "or malformed:\n",
                 paste(v0.4.3, collapse = "\n"), "\n\n")
    }

    ## If we are here, the baseDir doesn't match any of the known
    ## output structures.
    stop(msg)
}

#' @keywords internal
#' @noRd
.checkAlevinFryInputFiles_0.4.3 <- function(mapDir, permitDir, quantDir) {
    ## Raise an error if any of the required files is missing
    reqFiles <- c(file.path(permitDir, "all_freq.tsv"),
                  file.path(quantDir, "featureDump.txt"),
                  file.path(mapDir, "aux_info/meta_info.json"),
                  file.path(quantDir, "quant.json"),
                  file.path(mapDir, "cmd_info.json"),
                  file.path(quantDir, "alevin", "quants_mat_rows.txt"))
    missingFiles <- reqFiles[vapply(reqFiles,
                                    function(f) {
                                        !file.exists(f)
                                    }, TRUE)]
    if (length(missingFiles) > 0) {
        return(missingFiles)
    }

    ## Check that all required columns in featureDump.txt are present
    coln <- unlist(utils::read.delim(
        file.path(quantDir, "featureDump.txt"),
        header = FALSE, as.is = TRUE, sep = "\t", nrows = 1))
    if (!(all(c("CorrectedReads", "MappingRate", "DedupRate",
                "NumGenesOverMean", "MappedReads", "DeduplicatedReads",
                "NumGenesExpressed") %in% coln))) {
        missingFiles <- c(file.path(quantDir, "featureDump.txt"))
    }

    if (length(missingFiles) > 0) {
        return(missingFiles)
    }

    ## If we are here, the input is consistent with the expectation
    return(NULL)
}

#' @keywords internal
#' @noRd
.checkAlevinFryInputFiles_0.5.0 <- function(mapDir, permitDir, quantDir) {
    ## Raise an error if any of the required files is missing
    reqFiles <- file.path(permitDir, "generate_permit_list.json")
    if (!file.exists(reqFiles)) {
        return(reqFiles)
    }
    reqFiles <- c(reqFiles,
                  file.path(permitDir, "permit_freq.bin"),
                  file.path(quantDir, "featureDump.txt"),
                  file.path(mapDir, "aux_info/meta_info.json"),
                  file.path(quantDir, "quant.json"),
                  file.path(mapDir, "cmd_info.json"))

    permitinfo <- rjson::fromJSON(file = file.path(permitDir,
                                                   "generate_permit_list.json"))
    if (permitinfo$`permit-list-type` != "unfiltered") {
        reqFiles <- c(reqFiles,
                      file.path(permitDir, "all_freq.bin"))
    }
    missingFiles <- reqFiles[vapply(reqFiles,
                                    function(f) {
                                        !file.exists(f)
                                    }, TRUE)]
    if (length(missingFiles) > 0) {
        return(missingFiles)
    }

    ## Check that all required columns in featureDump.txt are present
    coln <- unlist(utils::read.delim(
        file.path(quantDir, "featureDump.txt"),
        header = FALSE, as.is = TRUE, sep = "\t", nrows = 1))
    if (!(all(c("CorrectedReads", "MappingRate", "DedupRate",
                "NumGenesOverMean", "MappedReads", "DeduplicatedReads",
                "NumGenesExpressed") %in% coln))) {
        missingFiles <- c(file.path(quantDir, "featureDump.txt"))
    }

    if (length(missingFiles) > 0) {
        return(missingFiles)
    }

    ## If we are here, the input is consistent with the expectation
    return(NULL)
}


#' @keywords internal
#' @noRd
.checkAlevinFryInputFiles_piscem0.6.0 <- function(mapDir, permitDir, quantDir) {
    ## Raise an error if any of the required files is missing
    reqFiles <- file.path(permitDir, "generate_permit_list.json")
    if (!file.exists(reqFiles)) {
        return(reqFiles)
    }
    reqFiles <- c(reqFiles,
                  file.path(permitDir, "permit_freq.bin"),
                  file.path(quantDir, "featureDump.txt"),
                  file.path(quantDir, "quant.json"),
                  file.path(mapDir, "map_info.json"))

    permitinfo <- rjson::fromJSON(file = file.path(permitDir,
                                                   "generate_permit_list.json"))
    if (permitinfo$`permit-list-type` != "unfiltered") {
        reqFiles <- c(reqFiles,
                      file.path(permitDir, "all_freq.bin"))
    }
    missingFiles <- reqFiles[vapply(reqFiles,
                                    function(f) {
                                        !file.exists(f)
                                    }, TRUE)]
    if (length(missingFiles) > 0) {
        return(missingFiles)
    }

    ## Check that all required columns in featureDump.txt are present
    coln <- unlist(utils::read.delim(
        file.path(quantDir, "featureDump.txt"),
        header = FALSE, as.is = TRUE, sep = "\t", nrows = 1))
    if (!(all(c("CorrectedReads", "MappingRate", "DedupRate",
                "NumGenesOverMean", "MappedReads", "DeduplicatedReads",
                "NumGenesExpressed") %in% coln))) {
        missingFiles <- c(file.path(quantDir, "featureDump.txt"))
    }

    if (length(missingFiles) > 0) {
        return(missingFiles)
    }

    ## If we are here, the input is consistent with the expectation
    return(NULL)
}

#' Summary plot of cell barcode collapsing
#'
#' Plot the original frequency of each cell barcode in the original whitelist
#' against the frequency after collapsing similar cell barcodes.
#'
#' @author Charlotte Soneson
#'
#' @param cbTable \code{data.frame} (such as the \code{cbTable} returned by
#'     \code{readAlevinQC} or \code{readAlevinFryQC}) with barcode frequencies
#'     before and after collapsing.
#' @param firstSelColName Character scalar indicating the name of the logical
#'     column in \code{cbTable} that corresponds to the original selection of
#'     barcodes for quantification.
#' @param countCol Character scalar indicating the name of the column in
#'     \code{cbTable} that corresponds to the collapsed barcode frequencies.
#'
#' @export
#'
#' @importFrom ggplot2 ggplot aes geom_abline geom_point theme_bw theme
#'     element_text xlab ylab geom_label
#' @importFrom rlang .data
#' @import dplyr
#'
#' @return A ggplot object
#'
#' @examples
#' alevin <- readAlevinQC(system.file("extdata/alevin_example_v0.14",
#'                                    package = "alevinQC"))
#' plotAlevinBarcodeCollapse(alevin$cbTable)
#'
plotAlevinBarcodeCollapse <- function(cbTable,
                                      firstSelColName = "inFirstWhiteList",
                                      countCol = "collapsedFreq") {
    ## Check input arguments
    .assertVector(x = cbTable, type = "data.frame")
    .assertScalar(x = firstSelColName, type = "character",
                  validValues = colnames(cbTable))
    .assertScalar(x = countCol, type = "character",
                  validValues = colnames(cbTable))
    .assertVector(x = cbTable[[firstSelColName]], type = "logical")
    stopifnot(all(c(countCol, "originalFreq") %in%
                      colnames(cbTable)))

    ## Get selected BCs and calculate the average read gain
    mrg <- cbTable %>% dplyr::filter(.data[[firstSelColName]]) %>%
        dplyr::summarize(
            mrg = signif(100 * mean(.data[[countCol]]/.data$originalFreq - 1,
                                    na.rm = TRUE), 4)) %>%
        dplyr::pull(mrg)

    ## Plot
    ggplot2::ggplot(cbTable %>% dplyr::filter(.data[[firstSelColName]]),
                    ggplot2::aes(x = .data$originalFreq,
                                 y = .data[[countCol]])) +
        ggplot2::geom_abline(slope = 1, intercept = 0) +
        ggplot2::geom_point() +
        ggplot2::theme_bw() +
        ggplot2::theme(axis.title = ggplot2::element_text(size = 12)) +
        ggplot2::xlab("Observed cell barcode frequency") +
        ggplot2::ylab("Cell barcode frequency, following reassignment") +
        ggplot2::geom_label(data = data.frame(x = -Inf, y = Inf),
                            aes(x = .data$x, y = .data$y,
                                label = paste0("Mean read gain per CB: ",
                                               mrg, "%")),
                            hjust = -0.05, vjust = 1.3)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

cpp_get_permit_freq_info <- function(fn) {
    .Call(`_alevinQC_cpp_get_permit_freq_info`, fn)
}

test_that("get_permit_freq works", {
    out <- cpp_get_permit_freq_info(system.file("extdata/permit_freq.bin",
                                                package = "alevinQC"))
    expect_type(out, "list")
    expect_length(out, 2)
    expect_equal(out[[1]], c("TGCTGCTAGACCACCT", "TGCTGCTAGACCACGC",
                             "TGCTGCTAGACCACCC", "TGCTGCTAGACCACGA",
                             "TGCTGCTAGACCACCG"))
    expect_equal(out[[2]], c(5, 5, 4, 5, 5))
})

#' Histogram of selected summary statistic
#'
#' @author Charlotte Soneson
#'
#' @param cbTable \code{data.frame} (such as the \code{cbTable} returned by
#'     \code{readAlevinQC} or \code{readAlevinFryQC}) containing the desired
#'     summary statistic in a column.
#' @param plotVar Character scalar giving the name of a numeric column of
#'     \code{cbTable} to plot.
#' @param axisLabel Character scalar giving the label of the selected
#'     statistic (will be displayed as the axis label in the plot).
#' @param colName Character scalar giving the name of a logical column of
#'     \code{cbTable} to use for filling the bars in the histogram.
#' @param cbName Character scalar giving the name of the set of barcodes
#'     defined by \code{colName}, used for labelling the plot legend.
#' @param firstSelColName Character scalar indicating the name of the logical
#'     column in \code{cbTable} that corresponds to the original selection of
#'     barcodes for quantification.
#'
#' @export
#'
#' @importFrom ggplot2 aes scale_fill_manual theme_bw theme
#'     labs ggplot geom_histogram
#' @import dplyr
#' @import rlang
#'
#' @return A ggplot object
#'
#' @examples
#' alevin <- readAlevinQC(system.file("extdata/alevin_example_v0.14",
#'                                    package = "alevinQC"))
#' plotAlevinHistogram(alevin$cbTable, plotVar = "dedupRate",
#'                     axisLabel = "Deduplication rate",
#'                     colName = "inFinalWhiteList",
#'                     cbName = "final whitelist")
#'
plotAlevinHistogram <- function(cbTable, plotVar = "dedupRate",
                                axisLabel = plotVar,
                                colName = "inFinalWhiteList",
                                cbName = "final whitelist",
                                firstSelColName = "inFirstWhiteList") {
    ## Check input arguments
    .assertVector(x = cbTable, type = "data.frame")
    .assertScalar(x = plotVar, type = "character",
                  validValues = colnames(cbTable))
    .assertVector(x = cbTable[[plotVar]], type = "numeric")
    .assertScalar(x = axisLabel, type = "character")
    .assertScalar(x = colName, type = "character",
                  validValues = colnames(cbTable))
    .assertVector(x = cbTable[[colName]], type = "logical")
    .assertScalar(x = cbName, type = "character")
    .assertScalar(x = firstSelColName, type = "character",
                  validValues = colnames(cbTable))
    .assertVector(x = cbTable[[firstSelColName]], type = "logical")

    ## Filter to only keep the quantified barcodes
    cbTable <- cbTable %>% dplyr::filter(.data[[firstSelColName]]) %>%
        dplyr::filter(!is.na(.data[[plotVar]]))
    nbins <- round(min(nrow(cbTable)/10, 100))
    ggplot2::ggplot(
        cbTable,
        ggplot2::aes(x = .data[[plotVar]],
                     fill = .data[[colName]])
    ) +
        ggplot2::geom_histogram(bins = nbins, color = "grey") +
        ggplot2::theme_bw() +
        ggplot2::scale_fill_manual(values = c(`TRUE` = "grey35",
                                              `FALSE` = "lightgrey")) +
        ggplot2::theme(legend.position = "bottom") +
        ggplot2::labs(x = axisLabel,
                      fill = paste0("Barcode included in ", cbName))
}

#' alevinQC
#'
#' alevinQC
#'
## usethis namespace: start
#' @useDynLib alevinQC, .registration = TRUE
#' @importFrom Rcpp sourceCpp
## usethis namespace: end
#' @keywords internal
"_PACKAGE"

globalVariables(c("V1", "V2", "V3", "V4", "V5", "CB",
                  "inFirstWhiteList", "inFinalWhiteList",
                  "MappingRate", "CorrectedReads",
                  "DedupRate", "NumGenesOverMean", "MappedReads",
                  "DeduplicatedReads", "NumGenesExpressed",
                  "collapsedFreq", "originalFreq", "nbrGenesAboveZero",
                  "totalUMICount", "ranking", "x"))

#' Panel of plots with quantification summary statistics
#'
#' @author Charlotte Soneson
#'
#' @param cbTable \code{data.frame} (such as the \code{cbTable} returned by
#'     \code{readAlevinQC} or \code{readAlevinFryQC}) with collapsed barcode
#'     frequencies, the total UMI count and the number of detected genes for
#'     each cell.
#' @param colName Character scalar giving the name of a logical column of
#'     \code{cbTable} to use for coloring the points.
#' @param cbName Character scalar giving the name of the set of barcodes
#'     defined by \code{colName}, used for labelling the plot legend.
#' @param firstSelColName Character scalar indicating the name of the logical
#'     column in \code{cbTable} that indicates the original selection of
#'     barcodes for quantification.
#'
#' @export
#'
#' @importFrom ggplot2 aes scale_color_manual scale_fill_manual theme_bw theme
#'     labs geom_point ggplot xlab ylab
#' @import dplyr
#' @importFrom cowplot plot_grid get_legend
#' @import rlang
#'
#' @return A ggplot object
#'
#' @examples
#' alevin <- readAlevinQC(system.file("extdata/alevin_example_v0.14",
#'                                    package = "alevinQC"))
#' plotAlevinQuant(alevin$cbTable, colName = "inFinalWhiteList",
#'                 cbName = "final whitelist")
#'
plotAlevinQuant <- function(cbTable, colName = "inFinalWhiteList",
                            cbName = "final whitelist",
                            firstSelColName = "inFirstWhiteList") {
    ## Check input arguments
    .assertVector(x = cbTable, type = "data.frame")
    .assertScalar(x = colName, type = "character",
                  validValues = colnames(cbTable))
    .assertVector(x = cbTable[[colName]], type = "logical")
    .assertScalar(x = cbName, type = "character")
    .assertScalar(x = firstSelColName, type = "character",
                  validValues = colnames(cbTable))
    .assertVector(x = cbTable[[firstSelColName]], type = "logical")
    stopifnot(all(c("collapsedFreq", "totalUMICount",
                    "nbrGenesAboveZero") %in%
                      colnames(cbTable)))

    ## Keep only CBs in first selection
    cbTable <- cbTable %>% dplyr::filter(.data[[firstSelColName]])

    ## Define shared ggplot layers
    gglayers <- list(
        ggplot2::geom_point(alpha = 0.5),
        ggplot2::scale_color_manual(values = c(`TRUE` = "red",
                                               `FALSE` = "darkgreen")),
        ggplot2::theme_bw()
    )

    ## Generate plots
    g1 <- ggplot2::ggplot(
        cbTable,
        ggplot2::aes(x = .data$collapsedFreq, y = .data$totalUMICount,
                     color = .data[[colName]])
    ) + gglayers +
        ggplot2::xlab("Barcode frequency") +
        ggplot2::ylab("Total UMI count")
    g2 <- ggplot2::ggplot(
        cbTable,
        ggplot2::aes(x = .data$collapsedFreq, y = .data$nbrGenesAboveZero,
                     color = .data[[colName]])
    ) + gglayers +
        ggplot2::xlab("Barcode frequency") +
        ggplot2::ylab("Number of detected genes")
    g3 <- ggplot2::ggplot(
        cbTable,
        ggplot2::aes(x = .data$totalUMICount, y = .data$nbrGenesAboveZero,
                     color = .data[[colName]])
    ) + gglayers +
        ggplot2::xlab("Total UMI count") +
        ggplot2::ylab("Number of detected genes")

    ## Generate combined plot
    cowplot::plot_grid(
        cowplot::plot_grid(
            g1 + ggplot2::theme(legend.position = "none"),
            g2 + ggplot2::theme(legend.position = "none"),
            g3 + ggplot2::theme(legend.position = "none"),
            nrow = 1, rel_widths = c(1, 1, 1)
        ),
        cowplot::get_legend(
            g1 + ggplot2::theme(legend.position = "bottom") +
                ggplot2::labs(color = paste0("Barcode included in ", cbName))
        ),
        ncol = 1, rel_heights = c(1, 0.1)
    )
}

---
output: 
    html_document:
        theme: spacelab
        toc: true
        toc_float: true
        df_print: kable
        code_folding: show
    pdf_document:
        toc: true
    BiocStyle::html_document:
        toc: true
    BiocStyle::pdf_document:
        toc: true
always_allow_html: yes
editor_options: 
    chunk_output_type: console
title: "`r sampleId`"
date: "`r Sys.Date()`"
author: "`r paste0('alevinQC (v', utils::packageVersion('alevinQC'), ')')`"
---

<style>
   tbody tr:nth-child(odd){
    background-color: #D6E0F5;
  }
</style>

```{r prep, include = FALSE}
knitr::opts_chunk$set(
    echo = showCode, 
    warning = FALSE, 
    message = FALSE,
    crop = NULL
)
knitr::opts_knit$set(
    progress = FALSE, 
    verbose = FALSE
)
```

```{r setup-alevin, echo = showCode && (quantMethod == "alevin"), eval = (quantMethod == "alevin")}
## Read input files
if (!quiet) message("Reading Alevin output files...")
alevin <- readAlevinQC(baseDir = baseDir, customCBList = customCBList)
firstSelColName <- "inFirstWhiteList"
```

```{r setup-alevinfry, echo = showCode && (quantMethod == "alevin-fry"), eval = (quantMethod == "alevin-fry")}
## Read input files
if (!quiet) message("Reading Alevin-fry output files...")
alevin <- readAlevinFryQC(mapDir = mapDir, permitDir = permitDir,
                          quantDir = quantDir)
firstSelColName <- "inPermitList"
```

# Version info for `r quantMethod` run

```{r, warning = FALSE}
suppressWarnings({
    knitr::kable(
        alevin$versionTable
    )
})
```

# Summary tables

## Full set of cell barcodes

```{r, warning = FALSE}
if (!quiet) message("Generating summary tables...")
suppressWarnings({
    knitr::kable(
        alevin$summaryTables$fullDataset
    )
})
```

```{r initial-whitelist-title, echo = FALSE, results = "asis", eval = ("initialWhitelist" %in% names(alevin$summaryTables))}
cat("## Initial whitelist")
```

```{r, warning = FALSE, echo = showCode && ("initialWhitelist" %in% names(alevin$summaryTables)), eval = ("initialWhitelist" %in% names(alevin$summaryTables))}
suppressWarnings({
    knitr::kable(
        alevin$summaryTables$initialWhitelist
    )
})
```

```{r final-whitelist-title, echo = FALSE, results = "asis", eval = ("finalWhitelist" %in% names(alevin$summaryTables))}
cat("## Final whitelist")
```

```{r, warning = FALSE, echo = showCode && ("finalWhitelist" %in% names(alevin$summaryTables)), eval = ("finalWhitelist" %in% names(alevin$summaryTables))}
suppressWarnings({
    knitr::kable(
        alevin$summaryTables$finalWhitelist
    )
})
```

```{r permitlist-title, echo = FALSE, results = "asis", eval = ("permitlist" %in% names(alevin$summaryTables))}
cat("## Permitlist")
```

```{r, warning = FALSE, echo = showCode && ("permitlist" %in% names(alevin$summaryTables)), eval = ("permitlist" %in% names(alevin$summaryTables))}
suppressWarnings({
    knitr::kable(
        alevin$summaryTables$permitlist
    )
})
```

```{r custom-cb-tables-title, echo = FALSE, results = 'asis', eval = (length(customCBList) > 0)}
cat("## Custom cell barcode set(s)")
```

```{r custom-cb-tables, echo = showCode && (length(customCBList) > 0), results = "asis", eval = (length(customCBList) > 0)}
for (cbl in names(customCBList)) {
    cat(paste0("### ", cbl))
    suppressWarnings({
        print(knitr::kable(
            alevin$summaryTables[[paste0("customCB__", cbl)]]
        ))
    })
    cat("\n")
}
```

# Knee plot

```{r, results="asis", echo=FALSE, eval=(quantMethod == "alevin")}
cat(paste0("The knee plot displays the number of times each cell barcode ", 
           "is observed, indecreasing order. By finding a 'knee' in this ", 
           "plot, Alevin determines a threshold (indicated in the plot) that ", 
           "defines an initial 'whitelist' - a set of cell barcodes that ", 
           "likely represent non-empty droplets - and distinguishes them from ", 
           "the background. The initial whitelisting is only performed if no ", 
           "external whitelist is provided when running alevin. In the ", 
           "figure below, red indicates cell barcodes in the initial ", 
           "whitelist, black indicates all other cell barcodes."))
```

```{r, results="asis", echo=FALSE, eval=(quantMethod == "alevin-fry")}
cat(paste0("The knee plot displays the number of times each cell barcode ", 
           "is observed, indecreasing order. By finding a 'knee' in this ", 
           "plot, alevin-fry determines a threshold (indicated in the plot) ", 
           "that defines a 'permitlist' - a set of cell barcodes that ", 
           "likely represent non-empty droplets - and distinguishes them from ", 
           "the background. In the figure below, red indicates cell barcodes ", 
           "in the permitlist, black indicates all other cell barcodes."))
```


```{r}
if (!quiet) message("Generating knee plot...")
plotAlevinKneeRaw(alevin$cbTable, firstSelColName = firstSelColName)
```

# Cell barcode error correction and merging with initial whitelist

```{r, results="asis", echo=FALSE, eval=(quantMethod == "alevin")}
cat(paste0("Once the initial set of whitelisted cell barcodes is defined, ", 
           "Alevin goes through the remaining cell barcodes. If a cell ", 
           "barcode is similar enough to a whitelisted cell barcode, it will ", 
           "be corrected and the reads will be added to those of the ", 
           "whitelisted one. The figure below shows the original frequency ", 
           "of the whitelisted barcodes vs the frequency after this ", 
           "correction. The reads corresponding to cell barcodes that can ", 
           "not be corrected to a whitelisted barcode are discarded."))
```

```{r, results="asis", echo=FALSE, eval=(quantMethod == "alevin-fry")}
cat(paste0("Once the initial permitlist of cell barcodes is defined, ", 
           "alevin-fry goes through the remaining cell barcodes. If a cell ", 
           "barcode is similar enough to cell barcode in the permitlist, it ", 
           "will be corrected and the reads will be added to those of the ", 
           "one in the permitlist. The figure below shows the original frequency ", 
           "of the barcodes in the permitlist vs the frequency after this ", 
           "correction. The reads corresponding to cell barcodes that can ", 
           "not be corrected to a barcode in the permitlist are discarded."))
```


```{r, echo = showCode && (quantMethod == "alevin"), eval = (quantMethod == "alevin")}
if (!quiet) message("Generating barcode collapsing plot...")
plotAlevinBarcodeCollapse(alevin$cbTable, firstSelColName = firstSelColName,
                          countCol = "collapsedFreq")
```

```{r, echo = showCode && (quantMethod == "alevin-fry"), eval = (quantMethod == "alevin-fry")}
if (!quiet) message("Generating barcode collapsing plot...")
plotAlevinBarcodeCollapse(alevin$cbTable, firstSelColName = firstSelColName,
                          countCol = "nbrMappedUMI")
```

# Quantification

```{r, results="asis", echo=FALSE, eval=(quantMethod == "alevin")}
cat(paste0("After cell barcode collapsing, Alevin estimates the UMI count ", 
           "for each cell and gene. Following quantification, an additional ", 
           "cell barcode whitelisting is performed with the aim of ", 
           "extracting good quality cells, using not only the barcode ", 
           "frequency but also other features such as the fraction of mapped ", 
           "reads, the duplication rate and the average gene count. The ", 
           "plots below show the association between the cell barcode ", 
           "frequency (the number of observed reads corresponding to a ", 
           "cell barcode), the total UMI count and the number of detected ", 
           "genes. The cell barcodes are colored by whether or not they ", 
           "are included in the final whitelist."))
```

```{r, results="asis", echo=FALSE, eval=(quantMethod == "alevin-fry")}
cat(paste0("After cell barcode collapsing, alevin-fry estimates the UMI count ",
           "for each cell and gene. The plots below show the association ", 
           "between the cell barcode frequency (the number of observed ", 
           "reads corresponding to a cell barcode), the total UMI count and ", 
           "the number of detected genes. "))
```

These figures can give an indication of whether the sequenced reads actually
align to genes, as well as the duplication rate and the degree of saturation.
For many droplet data sets, the association between the barcode frequency and
the total UMI count is rougly linear, while the association of any of these with
the number of detected genes often deviates from linearity, if a small subset of
the genes are assigned a large fraction of the UMI counts.

```{r fig.width = 10, fig.height = 4, eval = (quantMethod == "alevin"), echo = showCode && (quantMethod == "alevin")}
if (!quiet) message("Generating quantification summary plot...")
plotAlevinQuant(alevin$cbTable, colName = "inFinalWhiteList",
                cbName = "final whitelist",
                firstSelColName = firstSelColName)
```

```{r fig.width = 10, fig.height = 4, eval = (quantMethod == "alevin-fry"), echo = showCode && (quantMethod == "alevin-fry")}
if (!quiet) message("Generating quantification summary plot...")
plotAlevinQuant(alevin$cbTable, colName = "inPermitList",
                cbName = "permitlist",
                firstSelColName = firstSelColName)
```

```{r custom-cb-plots-title, echo = FALSE, results = 'asis', eval = (length(customCBList) > 0)}
cat("## Custom cell barcode set(s)")
```

```{r custom-cb-plots, echo = showCode && (length(customCBList) > 0), eval = (length(customCBList) > 0), fig.width = 10, fig.height = 4}
for (cbl in names(customCBList)) {
    print(plotAlevinQuant(alevin$cbTable, colName = paste0("customCB__", cbl),
                          cbName = cbl), firstSelColName = firstSelColName)
}
```

# Knee plot, number of detected genes

Similarly to the knee plot that was used to select the initial set of cell 
barcodes, the plot below shows the number of detected genes for each cell
barcode included in the 
`r ifelse(quantMethod == "alevin", "initial whitelist", "permitlist")`, 
in decreasing order.

```{r}
if (!quiet) message("Generating knee plot for nbr genes...")
plotAlevinKneeNbrGenes(alevin$cbTable, firstSelColName = firstSelColName)
```

# Selected summary distributions

The histograms below show the distributions of the deduplication rates 
(number of deduplicated UMI counts/number of mapped reads) and the 
mapping rates, across the cells retained in the 
`r ifelse(quantMethod == "alevin", "initial whitelist", "permitlist")`.

```{r, fig.width = 10, fig.height = 4, echo = showCode && (quantMethod == "alevin"), eval = (quantMethod == "alevin")}
if (!quiet) message("Generating summary distribution plots...")
cowplot::plot_grid(
    plotAlevinHistogram(alevin$cbTable, plotVar = "dedupRate",
                        axisLabel = "Deduplication rate",
                        colName = "inFinalWhiteList",
                        cbName = "final whitelist",
                        firstSelColName = firstSelColName),
    plotAlevinHistogram(alevin$cbTable, plotVar = "mappingRate",
                        axisLabel = "Mapping rate",
                        colName = "inFinalWhiteList",
                        cbName = "final whitelist",
                        firstSelColName = firstSelColName),
    nrow = 1
)
```

```{r, fig.width = 10, fig.height = 4, echo = showCode && (quantMethod == "alevin-fry"), eval = (quantMethod == "alevin-fry")}
if (!quiet) message("Generating summary distribution plots...")
cowplot::plot_grid(
    plotAlevinHistogram(alevin$cbTable, plotVar = "dedupRate",
                        axisLabel = "Deduplication rate",
                        colName = firstSelColName,
                        cbName = "permitlist",
                        firstSelColName = firstSelColName),
    plotAlevinHistogram(alevin$cbTable, plotVar = "mappingRate",
                        axisLabel = "Mapping rate",
                        colName = firstSelColName,
                        cbName = "permitlist",
                        firstSelColName = firstSelColName),
    nrow = 1
)
```

```{r custom-cb-hists-title, echo = FALSE, results = 'asis', eval = (length(customCBList) > 0)}
cat("## Custom cell barcode set(s)")
```

```{r custom-cb-hists, echo = showCode && (length(customCBList) > 0), eval = (length(customCBList) > 0), fig.width = 10, fig.height = 4}
for (cbl in names(customCBList)) {
    print(cowplot::plot_grid(
        plotAlevinHistogram(alevin$cbTable, plotVar = "dedupRate",
                            axisLabel = "Deduplication rate",
                            colName = paste0("customCB__", cbl),
                            cbName = cbl,
                            firstSelColName = firstSelColName),
        plotAlevinHistogram(alevin$cbTable, plotVar = "mappingRate",
                            axisLabel = "Mapping rate",
                            colName = paste0("customCB__", cbl),
                            cbName = cbl,
                            firstSelColName = firstSelColName),
        nrow = 1
    ))
}
```


# Session info

```{r}
sessionInfo()
```

---
title: "alevinQC"
author: "Charlotte Soneson"
date: "`r Sys.Date()`"
package: "alevinQC"
output: 
    BiocStyle::html_document
bibliography: bibliography.bib
abstract: >
    `r Biocpkg("alevinQC")` reads output files from alevin and 
    generates summary reports.
vignette: >
    %\VignetteIndexEntry{alevinQC}
    %\VignetteEncoding{UTF-8}  
    %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r v1, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = TRUE,
  crop = NULL
)
```

# Introduction

The purpose of the `r Biocpkg("alevinQC")` package is to generate a summary QC
report based on the output of an
[alevin](https://salmon.readthedocs.io/en/latest/alevin.html)
[@Srivastava2019-fz] run. The QC report can be generated as a html or pdf file,
or launched as a shiny application.

# Installation

`alevinQC` can be installed using the `BiocManager` CRAN package.

```{r install, eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("alevinQC")
```

After installation, load the package into the R session.

```{r v2, message=FALSE}
library(alevinQC)
```

Note that in order to process output from Salmon v0.14 or later, you need 
Alevin v1.1 or later. 

# Assumed output directory structure

For more information about running alevin, we refer to the
[documentation](https://salmon.readthedocs.io/en/latest/alevin.html). When
invoked, alevin generates several output files in the specified output
directory. `r Biocpkg("alevinQC")` assumes that this structure is retained, and
will return an error if it isn't  - thus, it is not recommended to move or
rename the output files from alevin. `r Biocpkg("alevinQC")` assumes that the
following files (in the indicated structure) are available in the provided
`baseDir` (note that currently, in order to generate the full set of files,
alevin must be invoked with the `--dumpFeatures` flag).

For alevin versions before 0.14:

```{r v3a, eval=FALSE, results='asis'}
baseDir
  |--alevin
  |    |--featureDump.txt
  |    |--filtered_cb_frequency.txt
  |    |--MappedUmi.txt
  |    |--quants_mat_cols.txt
  |    |--quants_mat_rows.txt
  |    |--quants_mat.gz
  |    |--raw_cb_frequency.txt
  |    |--whitelist.txt
  |--aux_info
  |    |--meta_info.json
  |--cmd_info.json
```

For alevin version 0.14 and later:

```{r v3b, eval=FALSE, results='asis'}
baseDir
  |--alevin
  |    |--featureDump.txt
  |    |--raw_cb_frequency.txt
  |    |--whitelist.txt  (depending on how alevin was run)
  |--aux_info
  |    |--meta_info.json
  |    |--alevin_meta_info.json
  |--cmd_info.json
```

# Check that all required alevin files are available

The report generation functions (see below) will check that all the required
files are available in the provided base directory. However, you can also call
the function `checkAlevinInputFiles()` to run the check manually. If one or more
files are missing, the function will raise an error indicating the missing
file(s).

```{r v4}
baseDir <- system.file("extdata/alevin_example_v0.14", package = "alevinQC")
checkAlevinInputFiles(baseDir = baseDir)
```

# Generate QC report

The `alevinQCReport()` function generates the QC report from the alevin output.
Depending on the file extension of the `outputFile` argument, and the value of
`outputFormat`, the function can generate either an html report or a pdf report.

```{r v5, message=FALSE, warning=FALSE}
outputDir <- tempdir()
alevinQCReport(baseDir = baseDir, sampleId = "testSample", 
               outputFile = "alevinReport.html", 
               outputFormat = "html_document",
               outputDir = outputDir, forceOverwrite = TRUE)
```

# Create shiny app

In addition to static reports, `r Biocpkg("alevinQC")` can also generate a shiny
application, containing the same summary figures as the pdf and html reports.

```{r v7, message=FALSE}
app <- alevinQCShiny(baseDir = baseDir, sampleId = "testSample")
```

Once created, the app can be launched using the `runApp()` function from the 
`r CRANpkg("shiny")` package.

```{r v8, eval=FALSE}
shiny::runApp(app)
```

It is possible to export the data used internally by the interactive 
application (in effect, the output from the internal call to 
`readAlevinQC()` or `readAlevinFryQC()`). To enable such export, first 
generate the `app` object as in the example above, and then assign the call 
to `shiny::runApp()` to a variable to capture the output. For example: 

```{r}
if (interactive()) {
    out <- shiny::runApp(app)
}
```

To activate the export, make sure to click the button 'Close app' in the top 
right corner in order to close the application (don't just close the window). 
This will take you back to your R session, where the variable `out` will be 
populated with the data used in the app.

# Generate individual plots

The individual plots included in the QC reports can also be independently
generated. To do so, we must first read the alevin output into an R object.

```{r v9}
alevin <- readAlevinQC(baseDir = baseDir)
```

The resulting list contains four entries:

- `cbTable`: a `data.frame` with various inferred characteristics of the
individual cell barcodes.
- `summaryTables`: a list of `data.frame`s with summary information about the
full data set, the initial set of whitelisted cells and the final set of
whitelisted cells, respectively.
- `versionTable`: a `matrix` with information about the invokation of alevin.
- `type`: a `character` scalar indicating how alevinQC interpreted the alevin
output directory.

```{r v10}
head(alevin$cbTable)
```

```{r v11}
knitr::kable(alevin$summaryTables$fullDataset)
knitr::kable(alevin$summaryTables$initialWhitelist)
knitr::kable(alevin$summaryTables$finalWhitelist)
```

```{r v12}
knitr::kable(alevin$versionTable)
```

The plots can now be generated using the dedicated plotting functions provided
with `r Biocpkg("alevinQC")` (see the help file for the respective function for
more information).

```{r v13, warning=FALSE}
plotAlevinKneeRaw(alevin$cbTable)
plotAlevinBarcodeCollapse(alevin$cbTable)
plotAlevinQuant(alevin$cbTable)
plotAlevinKneeNbrGenes(alevin$cbTable)
```

# Session info

```{r v14}
sessionInfo()
```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotAlevinBarcodeCollapse.R
\name{plotAlevinBarcodeCollapse}
\alias{plotAlevinBarcodeCollapse}
\title{Summary plot of cell barcode collapsing}
\usage{
plotAlevinBarcodeCollapse(
  cbTable,
  firstSelColName = "inFirstWhiteList",
  countCol = "collapsedFreq"
)
}
\arguments{
\item{cbTable}{\code{data.frame} (such as the \code{cbTable} returned by
\code{readAlevinQC} or \code{readAlevinFryQC}) with barcode frequencies
before and after collapsing.}

\item{firstSelColName}{Character scalar indicating the name of the logical
column in \code{cbTable} that corresponds to the original selection of
barcodes for quantification.}

\item{countCol}{Character scalar indicating the name of the column in
\code{cbTable} that corresponds to the collapsed barcode frequencies.}
}
\value{
A ggplot object
}
\description{
Plot the original frequency of each cell barcode in the original whitelist
against the frequency after collapsing similar cell barcodes.
}
\examples{
alevin <- readAlevinQC(system.file("extdata/alevin_example_v0.14",
                                   package = "alevinQC"))
plotAlevinBarcodeCollapse(alevin$cbTable)

}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotAlevinKneeNbrGenes.R
\name{plotAlevinKneeNbrGenes}
\alias{plotAlevinKneeNbrGenes}
\title{Knee plot of the number of detected genes per cell}
\usage{
plotAlevinKneeNbrGenes(cbTable, firstSelColName = "inFirstWhiteList")
}
\arguments{
\item{cbTable}{\code{data.frame} (such as the \code{cbTable} returned by
\code{readAlevinQC} or \code{readAlevinFryQC}) with the number of
detected genes per cell.}

\item{firstSelColName}{Character scalar indicating the name of the logical
column in \code{cbTable} that corresponds to the original selection of
barcodes for quantification.}
}
\value{
A ggplot object
}
\description{
Plot the number of detected genes per cell in decreasing order.
Only cells contained in the original whitelist are considered.
}
\examples{
alevin <- readAlevinQC(system.file("extdata/alevin_example_v0.14",
                                   package = "alevinQC"))
plotAlevinKneeNbrGenes(alevin$cbTable)

}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAlevinQC.R
\name{readAlevinQC}
\alias{readAlevinQC}
\title{Read alevin data required to generate summary report}
\usage{
readAlevinQC(baseDir, customCBList = list())
}
\arguments{
\item{baseDir}{Path to the output directory from the alevin run (should be
the directory containing the \code{alevin} directory).}

\item{customCBList}{Named list with custom set(s) of barcodes to provide
summary statistics/plots for, in addition to the whitelists generated by
alevin.}
}
\value{
A list collecting all necessary information for generating the
  summary report/shiny app.
}
\description{
Read all alevin output files required to generate the summary report or shiny
app.
}
\examples{
alevin <- readAlevinQC(system.file("extdata/alevin_example_v0.14",
                       package = "alevinQC"))

}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/alevinQCShiny.R
\name{qcShiny}
\alias{qcShiny}
\alias{alevinQCShiny}
\alias{alevinFryQCShiny}
\alias{simpleafQCShiny}
\title{Generate alevin/alevin-fry summary shiny app}
\usage{
alevinQCShiny(baseDir, sampleId, customCBList = list(), addStopButton = TRUE)

alevinFryQCShiny(mapDir, permitDir, quantDir, sampleId, addStopButton = TRUE)

simpleafQCShiny(simpleafQuantDir, sampleId, addStopButton = TRUE)
}
\arguments{
\item{baseDir}{(Only used for alevin output) Path to the output directory
from the alevin run (should be the directory containing the
\code{alevin} directory).}

\item{sampleId}{Sample ID, will be used set the title for the app.}

\item{customCBList}{Named list with custom set(s) of barcodes to provide
summary statistics/plots for, in addition to the whitelists generated by
alevin.}

\item{addStopButton}{Logical scalar. If \code{TRUE} (default), will add a
dropdown menu with a button to stop the app (by calling
\code{shiny::stopApp}) and return a list with the information displayed
in the app.}

\item{mapDir}{(Only used for alevin-fry output) Path to the output directory
from the salmon alevin run (should be the directory containing the
\code{map.rad} file).}

\item{permitDir}{(Only used for alevin-fry output) Path to the output
directory from the permit list generation step (should be
the directory containing the \code{all_freq.tsv} file).}

\item{quantDir}{(Only used for alevin-fry output) Path to the output
directory from the alevin-fry quantification step (should be
the directory containing the \code{alevin} directory).}

\item{simpleafQuantDir}{(Only used for simpleaf output) Path to the output
directory from the simpleaf run (should be the directory containing the
\code{af_map} and \code{af_quant} directories).}
}
\value{
A shiny app.
}
\description{
Generate a shiny app summarizing the main aspects of an alevin/alevin-fry
quantification run. The app generation assumes that alevin has been run
with the --dumpFeatures flag to generate the necessary output files.
}
\examples{
app <- alevinQCShiny(
    baseDir = system.file("extdata/alevin_example_v0.14",
                          package = "alevinQC"),
    sampleId = "example")
if (interactive()) {
    shiny::runApp(app)
}

app <- alevinFryQCShiny(
    mapDir = system.file("extdata/alevinfry_example_v0.5.0/map",
                         package = "alevinQC"),
    permitDir = system.file("extdata/alevinfry_example_v0.5.0/permit",
                            package = "alevinQC"),
    quantDir = system.file("extdata/alevinfry_example_v0.5.0/quant",
                           package = "alevinQC"),
    sampleId = "example")
if (interactive()) {
    shiny::runApp(app)
}

app <- simpleafQCShiny(
    simpleafQuantDir = system.file("extdata/alevinfry_example_piscem_v0.6.0",
                                   package = "alevinQC"),
    sampleId = "example")
if (interactive()) {
    shiny::runApp(app)
}

}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkAlevinInputFiles.R
\name{checkAlevinInputFiles}
\alias{checkAlevinInputFiles}
\title{Check that all required input files are available}
\usage{
checkAlevinInputFiles(baseDir)
}
\arguments{
\item{baseDir}{Path to the output directory from the alevin run (should be
the directory containing the \code{alevin} directory).}
}
\value{
Returns nothing, raises an error if any of the required files are
  missing.
}
\description{
Check that all required input files are available
}
\examples{
checkAlevinInputFiles(system.file("extdata/alevin_example_v0.14",
                                  package = "alevinQC"))

}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/alevinQC-package.R
\docType{package}
\name{alevinQC-package}
\alias{alevinQC}
\alias{alevinQC-package}
\title{alevinQC}
\description{
alevinQC
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/csoneson/alevinQC}
  \item Report bugs at \url{https://github.com/csoneson/alevinQC/issues}
}

}
\author{
\strong{Maintainer}: Charlotte Soneson \email{charlottesoneson@gmail.com} (\href{https://orcid.org/0000-0003-3833-2169}{ORCID})

Authors:
\itemize{
  \item Avi Srivastava
  \item Rob Patro
  \item Dongze He
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkAlevinFryInputFiles.R
\name{checkAlevinFryInputFiles}
\alias{checkAlevinFryInputFiles}
\title{Check that all required input files are available for alevin-fry}
\usage{
checkAlevinFryInputFiles(mapDir, permitDir, quantDir)
}
\arguments{
\item{mapDir}{Path to the directory containing the map.rad file}

\item{permitDir}{Path to the output directory of the alevin-fry
generate-permit-list command.}

\item{quantDir}{Path to the output of the alevin-fry quant command.}
}
\value{
Returns nothing, raises an error if any of the required files are
  missing.
}
\description{
Check that all required input files are available for alevin-fry
}
\examples{
checkAlevinFryInputFiles(
    mapDir = system.file("extdata/alevinfry_example_v0.5.0/map",
                         package = "alevinQC"),
    permitDir = system.file("extdata/alevinfry_example_v0.5.0/permit",
                            package = "alevinQC"),
    quantDir = system.file("extdata/alevinfry_example_v0.5.0/quant",
                           package = "alevinQC"))

}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotAlevinHistogram.R
\name{plotAlevinHistogram}
\alias{plotAlevinHistogram}
\title{Histogram of selected summary statistic}
\usage{
plotAlevinHistogram(
  cbTable,
  plotVar = "dedupRate",
  axisLabel = plotVar,
  colName = "inFinalWhiteList",
  cbName = "final whitelist",
  firstSelColName = "inFirstWhiteList"
)
}
\arguments{
\item{cbTable}{\code{data.frame} (such as the \code{cbTable} returned by
\code{readAlevinQC} or \code{readAlevinFryQC}) containing the desired
summary statistic in a column.}

\item{plotVar}{Character scalar giving the name of a numeric column of
\code{cbTable} to plot.}

\item{axisLabel}{Character scalar giving the label of the selected
statistic (will be displayed as the axis label in the plot).}

\item{colName}{Character scalar giving the name of a logical column of
\code{cbTable} to use for filling the bars in the histogram.}

\item{cbName}{Character scalar giving the name of the set of barcodes
defined by \code{colName}, used for labelling the plot legend.}

\item{firstSelColName}{Character scalar indicating the name of the logical
column in \code{cbTable} that corresponds to the original selection of
barcodes for quantification.}
}
\value{
A ggplot object
}
\description{
Histogram of selected summary statistic
}
\examples{
alevin <- readAlevinQC(system.file("extdata/alevin_example_v0.14",
                                   package = "alevinQC"))
plotAlevinHistogram(alevin$cbTable, plotVar = "dedupRate",
                    axisLabel = "Deduplication rate",
                    colName = "inFinalWhiteList",
                    cbName = "final whitelist")

}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotAlevinQuant.R
\name{plotAlevinQuant}
\alias{plotAlevinQuant}
\title{Panel of plots with quantification summary statistics}
\usage{
plotAlevinQuant(
  cbTable,
  colName = "inFinalWhiteList",
  cbName = "final whitelist",
  firstSelColName = "inFirstWhiteList"
)
}
\arguments{
\item{cbTable}{\code{data.frame} (such as the \code{cbTable} returned by
\code{readAlevinQC} or \code{readAlevinFryQC}) with collapsed barcode
frequencies, the total UMI count and the number of detected genes for
each cell.}

\item{colName}{Character scalar giving the name of a logical column of
\code{cbTable} to use for coloring the points.}

\item{cbName}{Character scalar giving the name of the set of barcodes
defined by \code{colName}, used for labelling the plot legend.}

\item{firstSelColName}{Character scalar indicating the name of the logical
column in \code{cbTable} that indicates the original selection of
barcodes for quantification.}
}
\value{
A ggplot object
}
\description{
Panel of plots with quantification summary statistics
}
\examples{
alevin <- readAlevinQC(system.file("extdata/alevin_example_v0.14",
                                   package = "alevinQC"))
plotAlevinQuant(alevin$cbTable, colName = "inFinalWhiteList",
                cbName = "final whitelist")

}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotAlevinKneeRaw.R
\name{plotAlevinKneeRaw}
\alias{plotAlevinKneeRaw}
\title{Knee plot of raw cell barcode frequencies}
\usage{
plotAlevinKneeRaw(cbTable, firstSelColName = "inFirstWhiteList")
}
\arguments{
\item{cbTable}{\code{data.frame} with raw barcode frequencies (such as the
\code{cbTable} returned by \code{readAlevinQC} or
\code{readAlevinFryQC}).}

\item{firstSelColName}{Character scalar indicating the name of the logical
column in \code{cbTable} that corresponds to the original selection of
barcodes for quantification.}
}
\value{
A ggplot object
}
\description{
Plot the raw cell barcode frequencies in decreasing order, and indicate a
predetermined breakpoint (indicating barcodes included in the original
whitelist) using color as well as a label.
}
\examples{
alevin <- readAlevinQC(system.file("extdata/alevin_example_v0.14",
                                   package = "alevinQC"))
plotAlevinKneeRaw(alevin$cbTable)

}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/alevinQCReport.R
\name{qcReport}
\alias{qcReport}
\alias{alevinQCReport}
\alias{simpleafQCReport}
\alias{alevinFryQCReport}
\title{Generate alevin/alevin-fry summary report}
\usage{
alevinQCReport(
  baseDir,
  sampleId,
  outputFile,
  outputDir = "./",
  outputFormat = NULL,
  showCode = FALSE,
  forceOverwrite = FALSE,
  knitrProgress = FALSE,
  quiet = FALSE,
  ignorePandoc = FALSE,
  customCBList = list(),
  ...
)

simpleafQCReport(
  simpleafQuantDir,
  sampleId,
  outputFile,
  outputDir = "./",
  outputFormat = NULL,
  showCode = FALSE,
  forceOverwrite = FALSE,
  knitrProgress = FALSE,
  quiet = FALSE,
  ignorePandoc = FALSE,
  customCBList = list(),
  ...
)

alevinFryQCReport(
  mapDir,
  permitDir,
  quantDir,
  sampleId,
  outputFile,
  outputDir = "./",
  outputFormat = NULL,
  showCode = FALSE,
  forceOverwrite = FALSE,
  knitrProgress = FALSE,
  quiet = FALSE,
  ignorePandoc = FALSE,
  customCBList = list(),
  ...
)
}
\arguments{
\item{baseDir}{(Only used for alevin output) Path to the output directory
from the alevin run (should be the directory containing the
\code{alevin} directory).}

\item{sampleId}{Sample ID, will be used to set the title for the report.}

\item{outputFile}{File name of the output report. The file name extension
must be either \code{.html} or \code{.pdf}, and consistent with the value
of \code{outputFormat}.}

\item{outputDir}{Path to the output directory where the report will be
generated.}

\item{outputFormat}{The format of the output report. Either
\code{"html_document"}, \code{"pdf_document"},
\code{"BiocStyle::html_document"} or \code{"BiocStyle::pdf_document"}.
The file name extension of \code{outputFile} must be consistent with this
choice.}

\item{showCode}{Logical, whether to display the R code in the report.}

\item{forceOverwrite}{Logical, whether to force overwrite an existing report
with the same name in the output directory.}

\item{knitrProgress}{Logical, whether to display the progress of \code{knitr}
when generating the report.}

\item{quiet}{Logical, whether to show progress messages.}

\item{ignorePandoc}{Logical, determines what to do if \code{pandoc} or
\code{pandoc-citeproc} is missing (if \code{Sys.which("pandoc")} or
\code{Sys.which("pandoc-citeproc")} returns ""). If \code{ignorePandoc}
is TRUE, only a warning is given. The figures will be generated, but not
the final report. If \code{ignorePandoc} is FALSE (default), the
execution stops immediately.}

\item{customCBList}{Named list with custom set(s) of barcodes to provide
summary statistics/plots for, in addition to the whitelists generated by
alevin.}

\item{...}{Other arguments that will be passed to \code{rmarkdown::render}.}

\item{simpleafQuantDir}{(Only used for simpleaf output) Path to the output
directory from the simpleaf run (should be the directory containing the
\code{af_map} and \code{af_quant} directories).}

\item{mapDir}{(Only used for alevin-fry output) Path to the output directory
from the salmon alevin run (should be the directory containing the
\code{map.rad} file).}

\item{permitDir}{(Only used for alevin-fry output) Path to the output
directory from the permit list generation step (should be
the directory containing the \code{all_freq.tsv} file).}

\item{quantDir}{(Only used for alevin-fry output) Path to the output
directory from the alevin-fry quantification step (should be
the directory containing the \code{alevin} directory).}
}
\value{
Generates a summary report in the \code{outputDir} directory, and
    returns (invisibly) the name of the generated report.
}
\description{
Generate a report summarizing the main aspects of an alevin/alevin-fry
quantification run. The report generation assumes that alevin/alevin-fry
has been run with the --dumpFeatures flag to generate the necessary
output files.
}
\details{
When the function is called, a .Rmd template file will be copied
    into the output directory, and \code{rmarkdown::render} will be called to
    generate the final report. If there is already a .Rmd file with the same
    name in the output directory, the function will raise an error and stop,
    to avoid overwriting the existing file. The reason for this behaviour is
    that the copied template in the output directory will be deleted once
    the report is generated.
}
\examples{
alevinQCReport(
    baseDir = system.file("extdata/alevin_example_v0.14",
                          package = "alevinQC"),
    sampleId = "example", outputFile = "alevinReport.html",
    outputDir = tempdir(), forceOverwrite = TRUE)

alevinFryQCReport(
    mapDir = system.file("extdata/alevinfry_example_v0.5.0/map",
                         package = "alevinQC"),
    permitDir = system.file("extdata/alevinfry_example_v0.5.0/permit",
                            package = "alevinQC"),
    quantDir = system.file("extdata/alevinfry_example_v0.5.0/quant",
                           package = "alevinQC"),
    sampleId = "example", outputFile = "alevinFryReport.html",
    outputDir = tempdir(), forceOverwrite = TRUE)

simpleafQCReport(
    simpleafQuantDir = system.file("extdata/alevinfry_example_piscem_v0.6.0",
                                   package = "alevinQC"),
    sampleId = "example", outputFile = "simpleafReport.html",
    outputDir = tempdir(), forceOverwrite = TRUE)

}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readAlevinFryQC.R
\name{readAlevinFryQC}
\alias{readAlevinFryQC}
\title{Read alevin-fry data required to generate summary report}
\usage{
readAlevinFryQC(mapDir, permitDir, quantDir)
}
\arguments{
\item{mapDir}{Path to the output directory from the \code{salmon alevin}
run (should be the directory containing the \code{alevin} folder).}

\item{permitDir}{Path to the output directory from the
\code{generate-permit-list} and \code{collate} runs.}

\item{quantDir}{Path to the output directory from the
\code{alevin-fry quant} run (should be the directory containing the
\code{alevin} folder).}
}
\value{
A list collecting all necessary information for generating the
    summary report/shiny app.
}
\description{
Read all alevin-fry output files required to generate the
summary report or shiny app.
}
\examples{
alevinfry <- readAlevinFryQC(
    mapDir = system.file("extdata/alevinfry_example_v0.5.0/map",
                         package = "alevinQC"),
    permitDir = system.file("extdata/alevinfry_example_v0.5.0/permit",
                            package = "alevinQC"),
    quantDir = system.file("extdata/alevinfry_example_v0.5.0/quant",
                           package = "alevinQC"))

}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotAlevinQuantPairs.R
\name{plotAlevinQuantPairs}
\alias{plotAlevinQuantPairs}
\title{Pairs plot with quantification summary statistics}
\usage{
plotAlevinQuantPairs(
  cbTable,
  colName = "inFinalWhiteList",
  firstSelColName = "inFirstWhiteList"
)
}
\arguments{
\item{cbTable}{\code{data.frame} (such as the \code{cbTable} returned by
\code{readAlevinQC} or \code{readAlevinFryQC}) with collapsed barcode
frequencies, the total UMI count and the number of detected genes
for each cell.}

\item{colName}{Character scalar giving the name of a logical column of
\code{cbTable} to use for coloring the points.}

\item{firstSelColName}{Character scalar indicating the name of the logical
column in \code{cbTable} that corresponds to the original selection of
barcodes for quantification.}
}
\value{
A ggmatrix object
}
\description{
Pairs plot with quantification summary statistics
}
\examples{
alevin <- readAlevinQC(system.file("extdata/alevin_example_v0.14",
                                   package = "alevinQC"))
plotAlevinQuantPairs(alevin$cbTable, colName = "inFinalWhiteList")

}
\author{
Charlotte Soneson
}
